// ***************************************************************************************
//                                          CONVERSATION
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static int count_fs_in_theme(struct DeniseHandle *handle,struct Theme *theme);

// ***************************************************************************************
// EXTERN              BOOL con_build_question(handle,AboutTheme,FromTheme)
// ***************************************************************************************
BOOL con_build_question(struct DeniseHandle *handle,struct Theme *AboutTheme,struct Theme *FromTheme)
{
	/*
	AboutTheme - podmiot, temat, na ktry zostanie zadane pytanie (NULL - zostanie wybrany losowo),
	FromTheme - temat, na podstawie ktorego zostanie zadane pytanie (NULL - zostanie wybrany losowo),

	algorytm:
	- jesli nie podano AboutTheme lub FromTheme to wybieramy losowo z bazy,
	- okreslamy jaki bedzie temat (podmiot) - Denise, USER, THEME, lub INNY
	- wybor faktu prostego:
	  szukamy faktu prostego, ktorego nie ma w temacie,
     algorytm:
     - pobieramy kolejne opcjonalne fakty proste A dla utworzenia pytania,
     - pobieramy kolejne fakty proste B z AboutTheme,
     - porownujemy oba fakty A i B bez podmiotow:
     		- jesli sa rozne (A_DIF_B) to wybieramy ten fakt prosty A dla utworzenia pytania,
     - jesli nie znaleziono odpowiedniego faktu prostego to rezygnujemy z zadania pytania,
   - przygotowujemy strukture odmiany,
   - odmieniamy zdanie i wypisujemy pytanie,
	*/

	static struct Declension decl;
   struct FactDatabase *factsA,*factsB;
   struct Fact *factA,*factB;
   struct FactSimple *fsA,*fsB,*fsBB=NULL;
   struct Theme *theme,*temp_about,*temp_from;
   int themes_number,nr1,nr2,trials,subject,write_subject;
   char temp[200];

   // ustalamy max. liczbe prob dla tworzenia pytania
   trials=3;

   temp_about=AboutTheme;
   temp_from=FromTheme;

	themes_number=0;
   for (factsA=handle->facts;factsA!=NULL;factsA=factsA->next)
   for (theme=factsA->themes;theme!=NULL;theme=theme->next)
   {
      if (theme->fwd_databaseID[0]=='\0') themes_number++;
   }

   if (themes_number<2)
   {
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_TOO_FEW_THEMES_FOR_QUESTION,"za malo tematow dla utworzenia pytania"));
		return TRUE;
   }

   BUILD_QUESTION:;
   if (trials==0) return FALSE;
   trials--;

	// jesli nie podano AboutTheme lub FromTheme to wybieramy losowo z bazy,
   if (AboutTheme==NULL)
   {
		for (nr2=0;nr2<100;nr2++) // limitowana petla
		{
			nr1=(rand() % themes_number);
		   for (factsA=handle->facts;factsA!=NULL;factsA=factsA->next)
		   for (AboutTheme=factsA->themes;AboutTheme!=NULL;AboutTheme=AboutTheme->next)
         {
            if (nr1==0) goto FOUNDED_ABOUTTHEME;
	         nr1--;
         }
         FOUNDED_ABOUTTHEME:;
			if (AboutTheme==NULL) continue;
         if (AboutTheme->fwd_databaseID[0]!='\0') continue;
         if (AboutTheme!=FromTheme) goto ABOUT_FOUNDED;
		}
   	AboutTheme=temp_about;
      FromTheme=temp_from;
      goto BUILD_QUESTION;

		ABOUT_FOUNDED:;
   }
	else
   {
	   if (AboutTheme->fwd_databaseID[0]!='\0')
      {
	      AboutTheme=ide_find_theme(handle,AboutTheme->fwd_databaseID,AboutTheme->fwd_number);
      }
   }
//DeniseReport(handle,"about theme: %s",AboutTheme->identify);

   if (FromTheme==NULL)
   {
		for (nr2=0;nr2<100;nr2++) // limitowana petla
		{
			nr1=(rand() % themes_number);
		   for (factsA=handle->facts;factsA!=NULL;factsA=factsA->next)
		   for (FromTheme=factsA->themes;FromTheme!=NULL;FromTheme=FromTheme->next)
         {
            if (nr1==0) goto FOUNDED_FROMTHEME;
	         nr1--;
         }
         FOUNDED_FROMTHEME:;
			if (FromTheme==NULL) continue;
         if (FromTheme->fwd_databaseID[0]!='\0') continue;
         if (AboutTheme!=FromTheme) goto FROM_FOUNDED;
      }
   	AboutTheme=temp_about;
      FromTheme=temp_from;
      goto BUILD_QUESTION;

		FROM_FOUNDED:;
   }
	else
   {
	   if (FromTheme->fwd_databaseID[0]!='\0')
      {
	      FromTheme=ide_find_theme(handle,FromTheme->fwd_databaseID,FromTheme->fwd_number);
      }
   }
//DeniseReport(handle,"from theme: %s",FromTheme->identify);

	// okreslamy jaki bedzie temat (podmiot) - Denise, USER, THEME, lub INNY
   if (AboutTheme==handle->cache.denise)
   {
   	subject=SUBJECT_DENISE;
      write_subject=(rand() % 5)==0 ? WRITE_WITH_PRONOUN : WRITE_WITHOUT_SUBJECT;
   }
   else if (AboutTheme==handle->cache.user)
   {
   	subject=SUBJECT_USER;
      write_subject=(rand() % 10)<6 ? WRITE_WITHOUT_SUBJECT : (rand() % 3)==0 ? WRITE_WITH_SUBJECT : WRITE_WITH_PRONOUN;
   }
   else if (AboutTheme==handle->cache.theme)
   {
   	subject=SUBJECT_THEME;
      write_subject=(rand() % 5)==0 ? WRITE_WITH_SUBJECT : WRITE_WITH_PRONOUN;
   }
   else
   {
   	subject=SUBJECT_OTHER;
		write_subject=WRITE_WITH_SUBJECT;
   }

	// wybor faktu prostego:
	// szukamy faktu prostego, ktorego nie ma w temacie,
   // pobieramy kolejne opcjonalne fakty proste A dla utworzenia pytania,
   for (factsA=handle->facts;factsA!=NULL;factsA=factsA->next)
   for (factA=factsA->facts;factA!=NULL;factA=factA->next)
   for (fsA=factA->fs;fsA!=NULL;fsA=fsA->next)
   {
      // pobieramy temat i sprawdzamy, czy to temat FromTheme
		if (ide_find_theme(handle,fsA->databaseID,fsA->theme_number)!=FromTheme) continue;

	   // pobieramy kolejne fakty proste B z AboutTheme,
	   for (factsB=handle->facts;factsB!=NULL;factsB=factsB->next)
	   for (factB=factsB->facts;factB!=NULL;factB=factB->next)
	   for (fsB=factB->fs;fsB!=NULL;fsB=fsB->next)
	   {
	      // pobieramy temat i sprawdzamy, czy to temat AboutTheme
			if (ide_find_theme(handle,fsB->databaseID,fsB->theme_number)!=AboutTheme) continue;

		   // porownujemy oba fakty A i B bez podmiotow:
			switch (log_fscmp(handle,fsA,fsB,CMP_WITHOUT_SUBJECT))
			{
			   // jesli sa rozne to wybieramy ten fakt prosty A dla utworzenia pytania,
				case FCMP_A_DIF_B:
            	fsBB=fsB;
					break;
				case FCMP_A_EQU_B:
				case FCMP_A_IN_B:
				case FCMP_B_IN_A:
   			case FCMP_A_PR_B:
   			case FCMP_B_PR_A:
   			case FCMP_A_EQU_B_NEG:
   			case FCMP_A_IN_B_NEG:
   			case FCMP_B_IN_A_NEG:
   			case FCMP_A_PR_B_NEG:
   			case FCMP_B_PR_A_NEG:
				default:
            	goto NEXT_FSA;
            	break;
			}
	   }
		goto FOUNDED_FSA;
      NEXT_FSA:;
   }
   // jesli nie znaleziono odpowiedniego faktu prostego to rezygnujemy z zadania pytania,
	// jest to blad
	if (trials==0)
   {
//   	DeniseReport(handle,"nie odnaleziono faktu prostego dla utworzenia pytania");
	   return FALSE;
   }
   else
   {
   	AboutTheme=temp_about;
      FromTheme=temp_from;
   	goto BUILD_QUESTION;
   }

	// znaleziono FSA
   FOUNDED_FSA:;

   // przygotowujemy strukture odmiany,
   // odmieniamy zdanie i wypisujemy pytanie,
//DeniseReport(handle,"%s<-%s",fsBB->sentence->base_words,fsA->sentence->base_words);

	gra_declension_clean(&decl);
/*	if (gra_subsent_declension(handle,fsBB->sentence,fsA->sentence,&decl,subject,write_subject)==FALSE)
   {
   	if (trials==0)
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PROBLEM_WITH_DECLENSION,"problem z odmiana zdania"));
         return FALSE;
      }
      else
      {
	   	AboutTheme=temp_about;
	      FromTheme=temp_from;
      	goto BUILD_QUESTION;
      }
   }
*/
	if (!fsBB)
		printf("Nie znaleziono drugiego faktu przy tworzeniu pytania!\n");
	else
		if (gra_subsent_declension(handle,fsBB->sentence,fsA->sentence,&decl,subject,write_subject)==FALSE)
		{
			if (trials==0)
			{
				DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PROBLEM_WITH_DECLENSION,"problem z odmiana zdania"));
				return FALSE;
			}
			else
			{
				AboutTheme=temp_about;
				FromTheme=temp_from;
				goto BUILD_QUESTION;
			}
		} 

	if (decl.results_number>1)
	{
   		nr1=rand() % decl.results_number;
   		sprintf(temp,"%s? (%d/%d)",decl.result[nr1],nr1+1,decl.results_number);
	}
	else
	{
   		sprintf(temp,"%s?",decl.result[0]);
	}

	DeniseMessage(handle,"%s",temp);

	return TRUE;
}

static int count_fs_in_theme(struct DeniseHandle *handle,struct Theme *theme)
{
   struct FactDatabase *facts;
   struct Fact *fact;
	struct FactSimple *fs;
	int number=0;

	for (facts=handle->facts;facts!=NULL;facts=facts->next)
   for (fact=facts->facts;fact!=NULL;fact=fact->next)
   for (fs=fact->fs;fs!=NULL;fs=fs->next)
   {
		if (ide_find_theme(handle,fs->databaseID,fs->theme_number)==theme) number++;
   }

   return number;
}

struct Theme *con_select_theme(struct DeniseHandle *handle,struct Theme *about_theme,const int type,int mode)
{
	/*
	funkcja odszukuje temat, zgodnie z podanym parametrem:
   gdzie theme to opcjonalny temat podmiotu (abouttheme),
   algorytmy wyboru tematu (wybor algorytmu zawarty w argumentach funkcji):
	- temat najbardziej podobny podmiotowi: (TYPE_ALIKE)
		- nie posiada faktow sprzecznych, (MODE_WITHOUT_NEG)
		- posiadajacy fakty sprzeczne, (MODE_WITH_NEG)
		- mogacy posiadac fakty sprzeczne, (MODE_FREE)
	- temat nie majacy zadnych faktow zgodnych z podmiotem: (TYPE_DIFFERENT)
		- nie posiada faktow sprzecznych, (MODE_WITHOUT_NEG)
		- posiadajacy fakty sprzeczne, (MODE_WITH_NEG)
		- mogacy posiadac fakty sprzeczne, (MODE_FREE)
	- temat najlepiej opisany - majacy najwiecej faktow, (TYPE_KNOWN)
	- temat najgorzej opisany - majacy najmniej faktow, (TYPE_UNKNOWN)
	- temat wybrany droga losowa, (TYPE_RANDOM)
	*/

   struct FactDatabase *facts,*factsA,*factsB;
   struct Fact *factA,*factB;
   struct FactSimple *fsA,*fsB;
   struct Theme *theme,*best_theme;
   int themes_number,random,fs_number,nr1,quality,best_quality;
   BOOL neg_exist;

   if ((type==TYPE_ALIKE || type==TYPE_DIFFERENT) && about_theme==NULL) return NULL;

	themes_number=0;
   for (facts=handle->facts;facts!=NULL;facts=facts->next)
   for (theme=facts->themes;theme!=NULL;theme=theme->next)
      if (theme->fwd_databaseID[0]=='\0') themes_number++;

   if (type==TYPE_RANDOM) random=(rand() % themes_number)+1;

   // cleaning...
   best_theme=NULL;
   best_quality=-1;
	fs_number=-1;

	// przeszukujemy wszystkie tematy
   for (facts=handle->facts;facts!=NULL;facts=facts->next)
   for (theme=facts->themes;theme!=NULL;theme=theme->next)
	{
      if (theme->fwd_databaseID[0]!='\0') continue;

      if (type==TYPE_RANDOM)
      {
      	random--;
         if (random==0) return theme;
		}
      else if (type==TYPE_KNOWN)
      {
			nr1=count_fs_in_theme(handle,theme);
         if (nr1>fs_number ||  fs_number==-1 || (nr1==fs_number && (rand() % 2)==0))
         {
         	best_theme=theme;
            fs_number=nr1;
         }
      }
      else if (type==TYPE_UNKNOWN)
      {
			nr1=count_fs_in_theme(handle,theme);
         if (nr1<fs_number ||  fs_number==-1|| (nr1==fs_number && (rand() % 2)==0))
         {
         	best_theme=theme;
            fs_number=nr1;
         }
      }
		else if (type==TYPE_ALIKE || type==TYPE_DIFFERENT)
      {
			neg_exist=FALSE;
         quality=0;

			// przeszukujemy wszystkie fs nalezace do theme
		   for (factsA=handle->facts;factsA!=NULL;factsA=factsA->next)
		   for (factA=factsA->facts;factA!=NULL;factA=factA->next)
		   for (fsA=factA->fs;fsA!=NULL;fsA=fsA->next)
		   {
		      // pobieramy temat i sprawdzamy, czy to temat theme
				if (ide_find_theme(handle,fsA->databaseID,fsA->theme_number)!=theme) continue;

			   // pobieramy kolejne fakty proste B z about_theme,
  			   for (factsB=handle->facts;factsB!=NULL;factsB=factsB->next)
			   for (factB=factsB->facts;factB!=NULL;factB=factB->next)
			   for (fsB=factB->fs;fsB!=NULL;fsB=fsB->next)
			   {
            	if (fsA==fsB) continue;

			      // pobieramy temat i sprawdzamy, czy to temat about_theme
					if (ide_find_theme(handle,fsB->databaseID,fsB->theme_number)!=about_theme) continue;

				   // porownujemy oba fakty A i B bez podmiotow:
					switch (log_fscmp(handle,fsA,fsB,CMP_WITHOUT_SUBJECT))
					{
						case FCMP_A_DIF_B:
							break;
                  // ponizsze fakty swiadcza o podobienstwach (czyzby?)
						case FCMP_A_EQU_B:
							quality++;
						case FCMP_A_IN_B:
						case FCMP_B_IN_A:
							quality++;
		   			case FCMP_A_PR_B:
		   			case FCMP_B_PR_A:
							quality++;
							break;
                  // ponizsze fakty swiadcza o sprzecznosciach (czyzby?)
		   			case FCMP_A_EQU_B_NEG:
		   			case FCMP_A_IN_B_NEG:
		   			case FCMP_B_IN_A_NEG:
		   			case FCMP_A_PR_B_NEG:
		   			case FCMP_B_PR_A_NEG:
                  	// jesli posiada fakty sprzeczne, a nie moze ich miec to pomijamy temat
				         if (mode==MODE_WITHOUT_NEG) goto NEXT_THEME;
							// posiada fakty sprzeczne i musi je meic, to zaznaczmy, ze sa
				         else if (mode==MODE_WITH_NEG) neg_exist=TRUE;
							break;
					}
			   }
		   }

			// jesli musi posiadac fakty sprzeczne, a ich nie ma to nastepny temat
			if (mode==MODE_WITH_NEG && neg_exist==FALSE) goto NEXT_THEME;

         // sprawdzenie, czy jest najbardziej podobny/rozny
			if (type==TYPE_ALIKE)
         {
         	if (quality>best_quality ||  best_quality==-1 || (quality==best_quality && (rand() % 2)==0))
            {
            	best_theme=theme;
               best_quality=quality;
            }
         }
			else if (type==TYPE_DIFFERENT)
         {
         	if (quality<best_quality || best_quality==-1 || (quality==best_quality && (rand() % 2)==0))
            {
            	best_theme=theme;
               best_quality=quality;
            }
         }
      }
      NEXT_THEME:;
   }

   if (type==TYPE_ALIKE)
   {
//   	DeniseReport(handle,"podobienstwo: %d",best_quality);
   }
   else if (type==TYPE_KNOWN || type==TYPE_UNKNOWN)
   {
//   	DeniseReport(handle,"znajomosc (liczba fs): %d",fs_number);
   }

	return best_theme;
}
