// ***************************************************************************************
//                                            MEANINGS
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static BOOL load_meanings(struct DeniseHandle* handle);

// ***************************************************************************************
// EXTERN                    BOOL dat_meanings_load(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_meanings_load(struct DeniseHandle* handle)
{
	handle->meanings=NULL;

	if (load_meanings(handle)==TRUE)
		return TRUE;

	return FAIL;
}

static BOOL load_meanings(struct DeniseHandle* handle)
{
	struct Meaning *mean=NULL,*mean_last=NULL;
	FILE *stream;
	char temp[200],temp1[200],temp2[200];
	int nr1;
	char *factsdirfile=NULL;

	asprintf(&factsdirfile,"%s%s%s",handle->Prefix,handle->FactsDir,MEANINGS_FILE);
	if ((stream=dat_file_open(handle,factsdirfile,"r"))==NULL) goto blad;

	// wczytujemy kolejne linie, pomijamy komentarze
	for (;;)
	{
		if (glo_fgets(temp,200,stream)==NULL) break;
		temp[strlen(temp)-1]='\0';

		if (temp[0]==DATABASE_COMMENT) continue;

		mean=(struct Meaning *) malloc(sizeof(struct Meaning));
		mean->next=NULL;

		glo_get_part(temp,1,temp1,'=');
		mean->basisA_number=glo_count_parts(handle,temp1,',');
		mean->basisA=(char **) malloc(mean->basisA_number*sizeof(char *));
		for (nr1=0;nr1<mean->basisA_number;nr1++)
		{
			glo_get_part(temp1,nr1+1,temp2,',');
			mean->basisA[nr1]=malloc(strlen(temp2)+1);
			memcpy(mean->basisA[nr1],temp2,strlen(temp2)+1);
//DeniseMessage(handle,"A!%s!",mean->basisA[nr1]);
		}

		glo_get_part(temp,2,temp1,'=');
		mean->basisB_number=glo_count_parts(handle,temp1,',');
		mean->basisB=(char **) malloc(mean->basisB_number*sizeof(char *));
		for (nr1=0;nr1<mean->basisB_number;nr1++)
		{
			glo_get_part(temp1,nr1+1,temp2,',');
			mean->basisB[nr1]=malloc(strlen(temp2)+1);
			memcpy(mean->basisB[nr1],temp2,strlen(temp2)+1);
//DeniseMessage(handle,"B!%s!",mean->basisB[nr1]);
		}

		// dodanie do listy
		if (handle->meanings==NULL) handle->meanings=mean;
		else mean_last->next=mean;
		mean_last=mean;
	}
	fclose(stream);
	if(factsdirfile) free(factsdirfile);
	return TRUE;

	blad:
	if(factsdirfile) free(factsdirfile);
	if (stream) fclose(stream);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),"meanings.dat");
//   return FAIL;
	return TRUE;
}

// ***************************************************************************************
// EXTERN                    BOOL dat_meanings_erase(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_meanings_erase(struct DeniseHandle* handle)
{
	struct Meaning *mean,*mean_next;

	for (mean=handle->meanings;mean!=NULL;mean=mean_next)
	{
		mean_next=mean->next;

		for (;mean->basisA_number>0;mean->basisA_number--)
			if (mean->basisA[mean->basisA_number-1]) free(mean->basisA[mean->basisA_number-1]);
		if (mean->basisA) free(mean->basisA);

		for (;mean->basisB_number>0;mean->basisB_number--)
			if (mean->basisB[mean->basisB_number-1]) free(mean->basisB[mean->basisB_number-1]);
		if (mean->basisB) free(mean->basisB);
		
		free(mean);
	}
	handle->meanings=NULL;

	return TRUE;
}

// ***************************************************************************************
// EXTERN                    BOOL dat_meanigs_save(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_meanings_save(struct DeniseHandle* handle)
{
	struct Meaning *mean;
	FILE *stream=NULL;
	int nr1;
	char *factsdirfile=NULL;

	if (handle->meanings==NULL) return TRUE;

	asprintf(&factsdirfile,handle->FactsDir,MEANINGS_FILE);
	if ((stream=dat_file_open(handle,factsdirfile,"w"))==NULL) return FAIL;

	for (mean=handle->meanings;mean!=NULL;mean=mean->next)
	{
		for (nr1=0;nr1<mean->basisA_number;nr1++)
		{
			fprintf(stream,"%s",mean->basisA[nr1]);
			if (nr1<mean->basisA_number-1) fprintf(stream,",");
		}
		fprintf(stream,"=");
		for (nr1=0;nr1<mean->basisB_number;nr1++)
		{
			fprintf(stream,"%s",mean->basisB[nr1]);
			if (nr1<mean->basisB_number-1) fprintf(stream,",");
		}
		fprintf(stream,"\n");
	}
	fclose(stream);
	if(factsdirfile) free(factsdirfile);
	return TRUE;
}
