// ***************************************************************************************
//                                          EXTRACTION
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static int get_fleksyjny_group(struct DeniseHandle *handle,struct FSentence *fsent,int zd_nr,int fleks_nr);

// ***************************************************************************************
// EXTERN                     int ext_set_denise_theme(handle)
// ***************************************************************************************
int ext_set_denise_theme(struct DeniseHandle* handle)
{
   /*
   funkcja przeszukuje wszystkie bazy faktow w poszukiwaniu tematu mowiacego o
   Denise, tj. wystepujacego w 2 osobie,

   funkcja zwraca liczbe takich tematow,
   a jesli ich liczba wynosi 1, to przypisuje wskaznik na ten temat do
   handle->cache.denise
   */

   struct FactDatabase *data;
   struct Theme *theme,*theme_founded;
   struct Fact *fact;
   struct FactSimple *fs;
   int ile=0;

   theme_founded=NULL;
   handle->cache.denise=NULL;

   for (data=handle->facts;data!=NULL;data=data->next)
   {
      for (fact=data->facts;fact!=NULL;fact=fact->next)
      {
         for (fs=fact->fs;fs!=NULL;fs=fs->next)
         {
            if (ext_get_person(handle,fs->sentence)!=2) continue;

            if ((theme=ide_find_theme(handle,fs->databaseID,fs->theme_number))==NULL) continue;
            if (ile==0) theme_founded=theme;
            else if (theme_founded==theme) continue;
            ile++;
         }
      }
   }

   if (ile==1)
	{
		handle->cache.denise=theme_founded;
	}

   return ile;
}

// ***************************************************************************************
// EXTERN                     int ext_get_person(handle,fsent)
// ***************************************************************************************
int ext_get_person(struct DeniseHandle* handle,struct FSentence *fsent)
{
   /*
   funkcja ma za zadanie wydobyc ze zdania informacje na temat osoby
   gramatycznej, ktora zostala w nim uzyta,

   algorytm:
   - pobieramy kolejne wyrazy fleksyjne,
      - pobieramy kolejne pozycje z bazy danych metod ekstrakcji osoby,
         - sprawdzamy czy wyraz fleksyjny jest zgodny z pozycja,
            - jesli TAK, to zwracamy osobe podawana na tej pozycji,
            - jesli NIE, to dalej,
   - zwracamy 0 w przypadku niepowodzenia,
   */

   char dane[100];
   int nr1,person;

   // pobieramy kolejne wyrazy fleksyjne,
   for (nr1=0;nr1<fsent->ile_fleks;nr1++)
   {
      glo_get_part(fsent->voc,nr1+1,dane,',');
		person=ext_get_person_from_word(handle,dane,fsent->function[get_fleksyjny_group(handle,fsent,0,nr1+1)-1]);
		if (person>0) return person;
   }

   return 0;
}

// ***************************************************************************************
// EXTERN                     int ext_get_person_from_word(handle,dane,function)
// ***************************************************************************************
int ext_get_person_from_word(struct DeniseHandle* handle,char *dane,int function)
{
	/*
	funkcja sprawdza, czy mozna zidentyfikowac osobe na podstawie podanej fleksji
	i funkcji - jesli mozna, to zwraca osobe, a jesli nie, to zwraca 0,

	algorytm:
   - pobieramy kolejne pozycje z bazy danych metod ekstrakcji osoby,
	   - sprawdzamy czy wyraz fleksyjny jest zgodny z pozycja,
   		- jesli TAK, to zwracamy osobe podawana na tej pozycji,
         - jesli NIE, to dalej,
	- w przypadku braku powodzenia zwracamy 0
	*/

   struct GrammarLo6Data *lo6;
   char posneg=TWIERDZENIE;

   // pobieramy kolejne pozycje z bazy danych metod ekstrakcji osoby,
   for (lo6=handle->language.lo6;lo6!=NULL;lo6=lo6->next)
   {
      // sprawdzamy czy wyraz fleksyjny jest zgodny z pozycja,
      if (lo6->func!=function) continue;
      if (gra_wyrazenie_check(handle,&posneg,lo6->czesc_mowy,dane)==0) continue;
      // jesli TAK, to zwracamy osobe podawana na tej pozycji,
      return lo6->person;
   }

	return 0;
}


static int get_fleksyjny_group(struct DeniseHandle *handle,struct FSentence *fsent,int zd_nr,int fleks_nr)
{
   // fleks_nr - numer (od 1) wyrazu fleksyjnego

   int nr1;

   if (fsent!=NULL)
   {
	   for (nr1=0;nr1<fsent->words_number;nr1++)
	      if (fsent->fleksyjny[nr1]==fleks_nr)
	         return fsent->member[nr1]; // zwracamy numer grupy (liczony od 1)

	   DeniseReport(handle,"blad!");
	   return -1; // blad!
   }
   else
   {
//   DeniseReport(handle,"nr: %d",                              handle->syntax.clause[zd_nr].words_number);
	   for (nr1=0;nr1<handle->syntax.clause[zd_nr].words_number;nr1++)
	      if (handle->syntax.clause[zd_nr].fleksyjny[nr1]==fleks_nr)
	         return handle->syntax.clause[zd_nr].member[nr1]; // zwracamy numer grupy (liczony od 1)

	   DeniseReport(handle,"wystapil blad!");
	   return -1; // blad!
   }

	return -1;
}

// ***************************************************************************************
// EXTERN                     ext_check_subject_person(handle,person,dane,podst)
// ***************************************************************************************
BOOL ext_check_subject_person(struct DeniseHandle *handle,int person,char *dane,char *podst)
{
	/*
	funkcja sprawdza, czy podany podmiot o danej fleksji (dane) i wyrazie
	podstawowym (podst) jest poprawnym podmiotem jesli zdanie jest w podanej
	osobie gramatycznej (person)

	dodatkowe oceny podmiotu w rozbiorze gramatycznym:
	- podmiot musi byc zgodny z osoba czasownika, z tematem, rozmowca, denise

   zasady:
	* 1 osoba (ja):
	- podmiot musi byc osobowy, wlasny, rozny od "denise"
	* 2 osoba (ty):
	- podmiotem musi byc temat denise
	* 3 osoba (on):
	- ?
	*/

   switch (person)
   {
   	// 1 osoba - JA (rozmowca)
   	case 1:
			//	podmiot musi byc osobowy, wlasny, rozny od "denise"

      	break;
      case 2: break;
      case 3: break;
   }

//	DeniseReport(handle,"osoba: %d, dane: %s, podst: %s\n",person,dane,podst);

	return TRUE;
}

// ***************************************************************************************
// EXTERN                     ext_sentence_filter(handle,sent)
// ***************************************************************************************
void ext_sentence_filter(struct DeniseHandle* handle,char *sent)
{
   /*
   funkcja filtruje wprowadzony ciag zgodnie z zasadami podanymi w pliku *.fil
   */

   struct FilterData *filter;
   char temp1[100],temp2[200],temp3[200],*ptr,len;
   int nr1,ile;

   if (sent==NULL || strlen(sent)==0) return;

	len=strlen(sent);
   for (ptr=sent;strlen(ptr)>0;ptr++)
   {
   	if (*ptr==' ' || *ptr==',' || *ptr=='!')
      {
			len=strlen(sent)-strlen(ptr);
         break;
      }
   }

   START:
   for (filter=handle->language.fil;filter!=NULL;filter=filter->next)
   {
      ile=glo_count_parts(handle,filter->find,'/');
      for (nr1=0;nr1<ile;nr1++)
      {
         glo_get_part(filter->find,nr1+1,temp1,'/');
         temp1[strlen(temp1)-1]='\0';
         memmove(temp1,temp1+1,strlen(temp1)+1);

         ptr=strstr(sent,temp1);
         if (ptr==NULL) continue;

         if (filter->range==2)
         {
				if ((int) strlen(sent)-(int) strlen(ptr)>len) continue;
         }

//printf("%s\n%s\n\n",ptr,sent);

         // jesli to podwojne spacje to pomijamy kontrole
         if (temp1[0]!=temp1[1] || temp1[0]!=' ')
         {
            // kontrola czy to rzeczywisty poczatek wyrazu a nie srodek
            if (strlen(ptr)<strlen(sent))
            {
//printf("LEFT: '%c'\n",*(ptr-1));
               if ((*(ptr-1))!=' ') continue;
            }
            // z prawej strony - koniec wyrazu
            if (strlen(ptr)>strlen(temp1))
            {
//printf("RIGHT: '%c'\n",*(ptr+strlen(temp1)));

               if ((*(ptr+strlen(temp1)))!=' ') continue;
            }
         }

         // kasowanie czesci
         if (filter->operation==1)
         {
            temp2[0]='\0';

            if (ptr!=sent)
            {
               sprintf(temp3,"%.*s",(int) strlen(sent)-strlen(ptr),sent);
               strcat(temp2,temp3);
            }
            if (strlen(ptr)>strlen(temp1)) strcat(temp2,ptr+strlen(temp1));

            if (temp2[0]==' ') strcpy(sent,temp2+1);
            else strcpy(sent,temp2);

            if (sent[strlen(sent)-1]==' ') sent[strlen(sent)-1]='\0';
         }
         // zamienienie czesci na argument
         else if (filter->operation==2)
         {
            temp2[0]='\0';

            if (ptr!=sent)
            {
               sprintf(temp3,"%.*s",(int) strlen(sent)-strlen(ptr),sent);
               strcat(temp2,temp3);
            }
            strcat(temp2,filter->replace);

            if (strlen(ptr)>strlen(temp1)) strcat(temp2,ptr+strlen(temp1));

            if (temp2[0]==' ') strcpy(sent,temp2+1);
            else strcpy(sent,temp2);
         }
         // wstawienie argumentu jako cale zdanie
         else if (filter->operation==3)
         {
            strcpy(sent,filter->replace);
            return;
         }

         goto START;
      }
   }

//printf("!%s!\n",sent);
}


// ***************************************************************************************
// EXTERN                     BOOL ext_pronoun_analysis(handle,zd_nr)
// ***************************************************************************************
BOOL ext_pronoun_analysis(struct DeniseHandle* handle,int zd_nr)
{
	/*
	<w przypadku jezyka polskiego:>
	tlumaczenie zaimkow:
	- rzeczownych sobowych (ja, ty, on)
	- przymiotnych dzierzawczych (moj, twoj, jego)

	tlumaczenie na: (zaleznie od osoby)
	1 osoba: rozmowca
	2 osoba: denise
	3 osoba: temat

	algorytm:
	- sprawdzenie, czy wyraz (fleksja) jest zaimkiem do zidentyfikowania
	- okreslenie obiektu, o ktory chodzi
	- jesli wyraz jest w grupie podmiotu, to kontrola zgodnosci z osoba gramatyczna:
   	- 1 osoba: dopuszczalny tylko rozmowca,
      - 2 osoba: dopuszczalna tylko denise,
      - 3 osoba: dopuszczalny tylko temat
	- wybranie optymalnego identyfikatora obiektu,
	- odmienienie go - znalezienie w bazie fleksji,
	- podmienienie wskaznikow w strukturze
	*/

	struct Fleksja *fleks,**fleksall;
	struct Theme *theme;
	struct GrammarLo8Data *lo8;
	int wyraz_nr,fleks_nr,poz,nr1,nr2,ile1,person,podm;
	char dane[100],dane_odm[100],temp[200],temp1[200],temp2[200],temp3[200],posneg=TWIERDZENIE,ident[100];

   person=0;

   // szukamy orzeczenia
   for (nr1=0;nr1<handle->syntax.clause[zd_nr].ile_fleks;nr1++)
	{
		nr2=get_fleksyjny_group(handle,NULL,zd_nr,nr1+1)-1;
	   if (handle->syntax.clause[zd_nr].function[nr2]==2)
      {
		   poz=handle->syntax.voc_buffer[handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]].poz[handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]];
		   if (poz!=-1) sprintf(dane,"%s%c",handle->syntax.voc_buffer[handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]].odm[handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]]->opis,poz+'A'-1);
		   else sprintf(dane,"%s",handle->syntax.voc_buffer[handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]].odm[handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]]->opis);
			person=ext_get_person_from_word(handle,dane,2);
         if (person!=0) break;
      }
   }

   // jesli nie mozna "wydobyc" osoby to wychodzimy z funkcji
	if (person==0) return FALSE;

   // szukamy grupy podmiotu
   podm=-1;
   for (nr1=0;nr1<handle->syntax.clause[zd_nr].groups_number;nr1++)
	{
	   if (handle->syntax.clause[zd_nr].function[nr1]==1)
      {
			podm=nr1;
			break;
      }
   }
	if (podm==-1) return FALSE;

//DeniseReport(handle,"BEGIN (%d i %d)",handle->syntax.clause[zd_nr].ile_fleks,handle->syntax.clause[zd_nr].groups_number);

	for (wyraz_nr=0;wyraz_nr<handle->syntax.buffer_len;wyraz_nr++)
	{
//DeniseReport(handle,"wac zaimka: %s",handle->syntax.voc_buffer[wyraz_nr].wyraz);
		for (fleks_nr=0;fleks_nr<handle->syntax.voc_buffer[wyraz_nr].fleks_ile;fleks_nr++)
		{
			// pobieramy pelny opis fleksyjny wyrazu
			poz=handle->syntax.voc_buffer[wyraz_nr].poz[fleks_nr];
      	if (poz!=-1) sprintf(dane,"%s%c",handle->syntax.voc_buffer[wyraz_nr].odm[fleks_nr]->opis,poz+'A'-1);
	      else sprintf(dane,"%s",handle->syntax.voc_buffer[wyraz_nr].odm[fleks_nr]->opis);

			// pobieramy kolejne pozycje z lo8
			for (lo8=handle->language.lo8;lo8!=NULL;lo8=lo8->next)
			{
				// sprawdzenie, czy wyraz (fleksja) jest zaimkiem do zidentyfikowania
				glo_get_part(lo8->cmp_data,1,temp,'+');
				if (gra_wyrazenie_check(handle,&posneg,temp,dane)==1)
				{
					// okreslenie obiektu, o ktory chodzi
					switch (lo8->person)
					{
						case 1: theme=handle->cache.user; break;
						case 2: theme=handle->cache.denise; break;
						case 3: theme=handle->cache.theme; break;
						default: theme=NULL;
					}

//DeniseReport(handle,"wac zaimka: %s, person-> %d (orzeczenie: %d) (theme==%s)",handle->syntax.voc_buffer[wyraz_nr].wyraz,lo8->person,person,theme==NULL ? "null" : "ok");
               if (theme==NULL) goto UNKNOWN;
               else if (theme!=NULL)
					{
//DeniseReport(handle,"theme not null");
						// jesli wyraz jest w grupie podmiotu, to kontrola zgodnosci z osoba gramatyczna:
						// sprawdzamy, czy wyraz_nr:fleks_nr jest w grupie podmiotu
                 	for (nr1=0;nr1<handle->syntax.clause[zd_nr].words_number;nr1++)
                  {
/*DeniseReport(handle,"%d=%d : %d=%d (%s=%s), member=%d (podm=%d)",
wyraz_nr,
handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1],
fleks_nr,
handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1],
handle->syntax.voc_buffer[wyraz_nr].wyraz,
handle->syntax.voc_buffer[handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]].wyraz,
handle->syntax.clause[zd_nr].member[nr1]-1,podm
);
*/                  	// czy wyraz jest w grupie podmiotu?
                     nr2=handle->syntax.clause[zd_nr].member[nr1]-1;
                     if (nr2!=podm) continue;

//DeniseReport(handle,"PODMIOT: wac zaimka: %s, lo8->person %d (person: %d)",handle->syntax.voc_buffer[wyraz_nr].wyraz,lo8->person,person);

							// czy to ten wyraz_nr?
                   	// czy to ten fleks_nr?
//DeniseReport(handle,"%d=%d : %d=%d",wyraz_nr,handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1],fleks_nr,handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]);
							if (wyraz_nr!=handle->syntax.clause[zd_nr].fleks_buff_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1] ||
								 fleks_nr!=handle->syntax.clause[zd_nr].fleks_nr[handle->syntax.clause[zd_nr].fleksyjny[nr1]-1]) continue;

//DeniseReport(handle,"1 !!! (%d - function: %d)",nr2,handle->syntax.clause[zd_nr].function[nr2]);

	                  // a jesli tak, to:
	                  switch (person)
	                  {
								// 1 osoba: dopuszczalny tylko rozmowca,
	                     case 1:
									if (theme!=handle->cache.user) goto UNKNOWN;
	                       	break;
								// 2 osoba: dopuszczalna tylko denise,
	                     case 2:
									if (theme!=handle->cache.denise) goto UNKNOWN;
	                      	break;
		               	// 3 osoba: dopuszczalny tylko temat
	                     case 3:
									if (theme!=handle->cache.theme) goto UNKNOWN;
	                       	break;
                     }
//							goto FOUNDED;
							break;
						}
                  // nie odnaleziono wyrazu w tym zdaniu - wiec pomijamy go
//                  goto NEXT;
//                  FOUNDED:;

//DeniseReport(handle,"person: %d",person);
//DeniseReport(handle,"zaimek: '%s', person: %d (lo8->person: %d), temat: %s, opis: %s\n",handle->syntax.voc_buffer[wyraz_nr].wyraz,person,lo8->person,theme->identify,lo8->cmp_data);

						// wybranie optymalnego identyfikatora obiektu,
						glo_get_part(theme->identify,1,ident,'+');

						// opisanie wymaganej fleksji
						glo_get_part(lo8->cmp_data,2,temp,'+');
						glo_get_part(lo8->cmp_data,1,temp3,'+');

						memset(dane_odm,'\0',99);
						ile1=glo_count_parts(handle,temp,',');
						for (nr1=0;nr1<ile1;nr1++)
						{
							glo_get_part(temp,nr1+1,temp1,',');
							if (temp1[0]=='$')
							{
								sprintf(temp2,"$%d",atoi(temp1+1));
								nr2=glo_get_part_position(1,temp3,temp2,',');
								dane_odm[nr1]=dane[nr2-1];
//								dane_odm[nr1]='!';
							}
							else if (temp1[0]=='?')
							{
								dane_odm[nr1]='?';
							}
							else
							{
								dane_odm[nr1]='A'-1+atoi(temp1);
							}
						}
  							// odmienienie go - znalezienie w bazie fleksji,
//printf("wyraz: %s, dane: %s\n",ident,dane_odm);

						fleksall=dat_find_word(handle,handle->language.vocabulary,ident,NULL,dane_odm,0);
						fleks=fleksall[0];
						free(fleksall);

						// podmienienie wskaznikow w strukturze
						if (fleks==NULL)
						{
						   DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WORD_DECLENSION_NOT_FOUND,"nie odnaleziono formy fleksyjnej: %s (%s)"),ident,dane_odm);
							goto UNKNOWN;
						}
						else
						{
							handle->syntax.voc_buffer[wyraz_nr].fleks[fleks_nr]=fleks;
							// UWAGA! to "waskie gardlo" - powinno byc bardziej ogolnie - lecz koszt jest zbyt duzy - lepiej napisac NOWA wersje
							handle->syntax.voc_buffer[wyraz_nr].odm[fleks_nr]=fleks->odm_head;
							if (glo_count_parts(handle,fleks->odm_head->odmiany,',')>0)
 							{
								if (dane_odm[strlen(dane_odm)-1]=='?')
								{
									// pierwszy lepszy... ale przeciez moze byc PUSTY? nalezaloby zrobic petle!
									handle->syntax.voc_buffer[wyraz_nr].poz[fleks_nr]=1;
//DeniseReport(handle,"founded MIANOWNIK\n");
								}
								else
								{
//DeniseReport(handle,"founded ZALEZNY: %d\n",dane_odm[strlen(dane_odm)-1]-'A'+1);
									handle->syntax.voc_buffer[wyraz_nr].poz[fleks_nr]=dane_odm[strlen(dane_odm)-1]-'A'+1;
								}
							}
							else handle->syntax.voc_buffer[wyraz_nr].poz[fleks_nr]=-1;
							break;
						}
					}
  	            goto NEXT;
					UNKNOWN:
					if (handle->viewparams.syntax==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PRONOUN_NOT_IDENTIFIED,"nie potrafie zidentyfikowac zaimka: %s"),handle->syntax.voc_buffer[wyraz_nr].wyraz);
					return FALSE;
				}
			}
         NEXT:;
		}
	}

	return TRUE;
}
