// ***************************************************************************************
//                                     IDENTIFY - THEMES
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static BOOL exist_id_in_fs(struct DeniseHandle* handle,char *ident,struct FactSimple *fs,int mode);
static BOOL exist_id_in_theme(struct DeniseHandle* handle,char *ident,struct Theme *theme,int mode);
static BOOL theme2_in_theme1(struct DeniseHandle* handle,struct Theme *theme1,struct Theme *theme2,int mode);

// ***************************************************************************************
// EXTERN                 int ide_find_theme_of_fs(handle,identifiers,mode)
// ***************************************************************************************
int ide_find_theme_of_fs(struct DeniseHandle* handle,struct ThemesList **result,struct FactSimple *fs,int mode)
{
   /*
   funkcja stara sie odnalezc w bazie temat odpowiadajacy podanemu faktowi prostemu
   
   mode:
   MODE_EXACT - bez uwzgledniania znaczenia kwantyfikatorow,
   MODE_COMPATIBLE - z uwzglednianiem znaczenia kwantyfikatorow,

   funkcja zwraca:
   IDENTIFIED_ONE - jesli znaleziono w bazie jeden temat
   IDENTIFIED_NOTHING - jesli w bazie nie ma zadnego odpowiedniego tematu
   IDENTIFIED_MORE_SAME - jesli w bazie jest wiele tematow z dokladnie takimi identyfikatorami glownymi,
                          (potrzebny jest dokladniejszy opis w postaci identyfikatorow podrzednych)
   IDENTIFIED_MORE_ALIKE - jesli w bazie jest jedynie wiele tematow podobnych, tj. do ktorych
                           pasuje temat podany w fakcie, ale nie pokrywa on wszystkich identyfikatorow glownych
                           (dotyczy tylko tych faktow prostych, ktore wykorzystuja do
                           identyfikacji nazwy wlasne wymagajace scislej identyfikacji)

   w result sa wszystkie tematy, ktore funkcja znalazla
   UWAGA! funkcja po prostu DOPISUJE do listy tematy, nie jest odpowiedzialna za ich kasowanie

   warunki poprawnej identyfikacji:
   - musza zostac zidentyfikowane wszystkie identyfikatory glowne danego tematu,
     aby zostal rozpoznany jako temat identyfikowanego faktu,
   - musza zostac zidentyfikowane wszystkie identyfikatory faktu (glowne i
     podrzedne) jako identyfikatory tematu (glowne lub podrzedne),

   algorytm:
   - rozpoznajemy, czy fakt wymaga scislej identyfikacji,
   - pobieramy kolejne tematy,
      <identyfikujemy wszystkie identyfikatory faktu>
      - pobieramy kolejne identyfikatory glowne faktu,
         - sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
           (moga to byc identyfikatory glowne lub podrzedne),
            - jesli NIE:
               - jesli to identyfikator zanegowany to kontynuuj,
               - jesli to identyfikator zwyczajny to pobieramy nastepny temat,
            - jesli TAK:
               - jesli to identyfikator zanegowany to pobieramy nastepny temat,
               - jesli to identyfikator zwyczajny to kontynuuj,
      - pobieramy kolejne identyfikatory podrzedne faktu,
         - sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
           (moga to byc identyfikatory glowne lub podrzedne),
            - jesli NIE:
               - jesli to identyfikator zanegowany to kontynuuj,
               - jesli to identyfikator zwyczajny to pobieramy nastepny temat,
            - jesli TAK:
               - jesli to identyfikator zanegowany to pobieramy nastepny temat,
               - jesli to identyfikator zwyczajny to kontynuuj,
      - jesli identyfikacja scisla oraz (IDENTIFIED_NOTHING lub IDENTIFIED_MORE_ALIKE), to:
         - jesli IDENTIFIED_NOTHING, to:
            - ustawiamy tryb IDENTIFIED_MORE_ALIKE,
            - zapamietujemy aktualna pozycje na liscie tematow,
         - zapamietujemy ten temat,
      <identyfikujemy glowne identyfikatory tematu>
      - pobieramy kolejne identyfikatory glowne tematu,
         - sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach faktu,
           (moga to byc identyfikatory glowne lub podrzedne),
            - jesli NIE:
               - jesli to identyfikator zanegowany to kontynuuj,
               - jesli to identyfikator zwyczajny to pobieramy nastepny temat,
            - jesli TAK:
               - jesli to identyfikator zanegowany to pobieramy nastepny temat,
               - jesli to identyfikator zwyczajny to kontynuuj,
      - jesli IDENTIFIED_MORE_ALIKE, to:
         - kasujemy wszystkie tematy, ktore dodano w tym trybie,
         - ustawiamy IDENTIFIED_NOTHING,
      - jesli bylo IDENTIFIED_NOTHING to ustawiamy IDENTIFIED_ONE,
        a w przeciwnym razie IDENTIFIED_MORE_SAME,
      - dodajemy temat do listy,
   */

   struct GrammarData *gr_data;
   struct FactDatabase *database;
   struct Theme *theme;
   struct ThemesList *themes_list,*themes_ptr=NULL;
   char posneg=TWIERDZENIE;
   char temp1[200],fleks[200];
   int ile1,nr1,decision;
   BOOL precision_identify=FALSE;

   decision=IDENTIFIED_NOTHING;

   // rozpoznajemy, czy fakt wymaga scislej identyfikacji,
   for (ile1=glo_count_parts(handle,fs->sentence->id_subject,','),nr1=0;nr1<ile1;nr1++)
   {
      glo_get_part(fs->sentence->id_subject,nr1+1,temp1,',');
      glo_get_part(temp1,2,fleks,'+');

      for (gr_data=handle->language.lo7;gr_data!=NULL;gr_data=gr_data->next)
      {
         if (gra_wyrazenie_check(handle,&posneg,gr_data->data,fleks)==1)
         {
            // wymagana scisla identyfikacja
            precision_identify=TRUE;
            goto POJDZ_SOBIE_DALEJ;
         }
      }
   }

   POJDZ_SOBIE_DALEJ:
   // <odszukanie tematu, ktory zawiera chocby jeden z identyfikatorow faktu>
   // pobieramy kolejne tematy,
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      for (theme=database->themes;theme!=NULL;theme=theme->next)
      {
         // pomijamy tematy forwardowane, gdyz zanalizujemy tematy, na ktore wskazuja,
         if (theme->fwd_databaseID[0]!='\0') continue;

         // <identyfikujemy wszystkie identyfikatory faktu>
         //  pobieramy kolejne identyfikatory glowne faktu,
         for (ile1=glo_count_parts(handle,fs->sentence->id_subject,','),nr1=0;nr1<ile1;nr1++)
         {
            glo_get_part(fs->sentence->id_subject,nr1+1,temp1,',');
            // sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
            // (moga to byc identyfikatory glowne lub podrzedne),
            switch (exist_id_in_theme(handle,temp1[0]=='!' ? temp1+1 : temp1,theme,mode))
            {
               // jesli NIE:
               case FALSE:
                  // jesli to identyfikator zanegowany to kontynuuj,
                  // jesli to identyfikator zwyczajny to pobieramy nastepny temat,
                  if (temp1[0]!='!') goto NEXT_THEME;
                  break;
               // jesli TAK:
               case TRUE:
                  // jesli to identyfikator zanegowany to pobieramy nastepny temat,
                  if (temp1[0]=='!') goto NEXT_THEME;
                  // jesli to identyfikator zwyczajny to kontynuuj,
                  break;
            }
         }

         // pobieramy kolejne identyfikatory podrzedne faktu,
         // sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
         // (moga to byc identyfikatory glowne lub podrzedne),
//            - jesli NIE:
//               - jesli to identyfikator zanegowany to kontynuuj,
//               - jesli to identyfikator zwyczajny to pobieramy nastepny temat,
//            - jesli TAK:
//               - jesli to identyfikator zanegowany to pobieramy nastepny temat,
//               - jesli to identyfikator zwyczajny to kontynuuj,
//         if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,"brak obslugi identyfikatorow podrzednych w fakcie");

         // jesli identyfikacja scisla oraz (IDENTIFIED_NOTHING lub IDENTIFIED_MORE_ALIKE), to:
         if (precision_identify==TRUE && (decision==IDENTIFIED_NOTHING || decision==IDENTIFIED_MORE_ALIKE))
         {
            // jesli IDENTIFIED_NOTHING, to:
            if (decision==IDENTIFIED_NOTHING)
            {
               // ustawiamy tryb IDENTIFIED_MORE_ALIKE,
               decision=IDENTIFIED_MORE_ALIKE;
            }
				// (Wczesniej te 3 linie byly w gornym if
            // zapamietujemy aktualna pozycje na liscie tematow,
            if ((*result)!=NULL) themes_ptr=(*result);
            else themes_ptr=NULL;

            // zapamietujemy ten temat,
            themes_list=(struct ThemesList *) malloc(sizeof(struct ThemesList));
            themes_list->theme=theme;
            themes_list->next=NULL;

            if (*result==NULL)
            {
            	*result=themes_list;
            }
            else
            {
					if (themes_ptr==NULL) DeniseReport(handle,"katastrofa... identify2.c kata strofa...");
               themes_list->next=themes_ptr->next;
               themes_ptr->next=themes_list;
            }
         }

         // <identyfikujemy glowne identyfikatory tematu>
         // pobieramy kolejne identyfikatory glowne tematu,
         for (ile1=glo_count_parts(handle,theme->identify,','),nr1=0;nr1<ile1;nr1++)
         {
            glo_get_part(theme->identify,nr1+1,temp1,',');

            // sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach faktu,
            // (moga to byc identyfikatory glowne lub podrzedne),
            switch (exist_id_in_fs(handle,temp1[0]=='!' ? temp1+1 : temp1,fs,mode))
            {
               // jesli NIE:
               case FALSE:
                  // jesli to identyfikator zanegowany to kontynuuj,
                  // jesli to identyfikator zwyczajny to pobieramy nastepny temat,
                  if (temp1[0]!='!') goto NEXT_THEME;
                  break;
               // jesli TAK:
               case TRUE:
                  // jesli to identyfikator zanegowany to pobieramy nastepny temat,
                  if (temp1[0]=='!') goto NEXT_THEME;
                  // jesli to identyfikator zwyczajny to kontynuuj,
                  break;
            }
         }

         // jesli IDENTIFIED_MORE_ALIKE, to:
         if (decision==IDENTIFIED_MORE_ALIKE)
         {
            // kasujemy wszystkie tematy, ktore dodano w tym trybie,
            if (themes_ptr==(*result) && (*result)!=NULL)
            {
               ide_themes_list_free((*result));
               (*result)=NULL;
            }
            else if (themes_ptr!=NULL)
            {
               ide_themes_list_free(themes_ptr->next);
               themes_ptr->next=NULL;
            }

            // ustawiamy IDENTIFIED_NOTHING,
            decision=IDENTIFIED_NOTHING;
         }

         // jesli bylo IDENTIFIED_NOTHING to ustawiamy IDENTIFIED_ONE,
         // a w przeciwnym razie IDENTIFIED_MORE_SAME,
         if (decision==IDENTIFIED_NOTHING) decision=IDENTIFIED_ONE;
         else if (decision==IDENTIFIED_ONE) decision=IDENTIFIED_MORE_SAME;

         // dodajemy temat do listy,
         themes_list=malloc(sizeof(struct ThemesList));
         themes_list->theme=theme;
         themes_list->next=NULL;

         if (*result==NULL) *result=themes_list;
         else
         {
            themes_list->next=(*result)->next;
            (*result)->next=themes_list;
         }

         NEXT_THEME:;
      }
   }

   return decision;
}

static BOOL exist_id_in_fs(struct DeniseHandle* handle,char *ident,struct FactSimple *fs,int mode)
{
   /*
   funkcja sprawdza, czy podany identyfikator zawiera sie w fakcie prostym fs
   zawiera sie - tzn. jest identyczny, lub jest sprecyzowaniem (A_EQU_B,A_PR_B,A_IN_B)

   zwraca:
   TRUE - jesli tak,
   FALSE - jesli nie,

   mode:
   MODE_EXACT - bez uwzgledniania znaczenia kwantyfikatorow,
   MODE_COMPATIBLE - z uwzglednianiem znaczenia kwantyfikatorow,

   algorytm:
   - pobieramy kolejne identyfikatory wyrazowe (glowne i podrzedne) faktu prostego,
      - porownujemy, czy podany identyfikator zawiera sie w podanym:
         - jesli TAK, to return TRUE,
         - jesli NIE, to kontynuujemy,
   */

   char temp1[100],*ptr,wyraz1[100],fleks1[100],wyraz2[100],fleks2[100];
   int ile1,nr1;

   glo_get_part(ident,1,wyraz1,'+');
   glo_get_part(ident,2,fleks1,'+');

   // pobieramy kolejne identyfikatory wyrazowe (glowne i podrzedne) faktu prostego,
   for (ptr=fs->sentence->id_subject,ile1=glo_count_parts(handle,ptr,','),nr1=0;nr1<ile1;nr1++)
   {
      glo_get_part(ptr,nr1+1,temp1,',');
      glo_get_part(temp1,1,wyraz2,'+');
      glo_get_part(temp1,2,fleks2,'+');

      // jesli to ostatni identyfikator glowny, to bierzemy teraz podrzedne
      if (nr1==ile1-1 && ptr==fs->sentence->id_subject && fs->sentence->id_global!=NULL && strlen(fs->sentence->id_global)>0)
      {
         ptr=fs->sentence->id_global;
         ile1=glo_count_parts(handle,ptr,',');
         nr1=-1;
      }

      // porownujemy, czy podany identyfikator zawiera sie w podanym:
      // jesli sa rowne (wyrazy) to RETURN TRUE,
      if (strcmp(wyraz1,wyraz2)==0) return TRUE;

      // jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
      if (mode==MODE_EXACT) continue;

      switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
      {
         case FCMP_A_EQU_B:
         case FCMP_A_IN_B:
         case FCMP_A_PR_B: return TRUE;
      }
   }

   return FALSE;
}

static BOOL exist_id_in_theme(struct DeniseHandle* handle,char *ident,struct Theme *theme,int mode)
{
   /*
   funkcja sprawdza, czy podany identyfikator zawiera sie w temacie,
   zawiera sie - tzn. jest identyczny, lub jest sprecyzowaniem (A_EQU_B,A_PR_B,A_IN_B),
   sprawdzane sa wszystkie identyfikatory podrzedne wszystkich faktow prostych
   podlegajacych temu tematowi,

   zwraca:
   TRUE - jesli tak,
   FALSE - jesli nie,

   mode:
   MODE_EXACT - bez uwzgledniania znaczenia kwantyfikatorow,
   MODE_COMPATIBLE - z uwzglednianiem znaczenia kwantyfikatorow,

   algorytm:
   - pobieramy kolejne identyfikatory glowne tematu,
      - porownujemy, czy podany identyfikator zawiera sie w pobranym:
         - jesli TAK, to return TRUE,
         - jesli NIE, to kontynuujemy,
   - pobieramy kolejne fakty proste z wszystkich baz faktow, ktore naleza do tego tematu,
      - porownujemy, czy podany identyfikator zawiera sie w pobranym fakcie prostym:
         - jesli TAK, to return TRUE,
         - jesli NIE, to kontynuujemy,
   */

   struct FactDatabase *database;
   struct Fact *fact;
   struct FactSimple *fs;
   char temp1[100],wyraz1[100],fleks1[100],wyraz2[100],fleks2[100];
   int ile1,nr1;

   // jesli temat jest forwardowany, to odszukujemy ostateczny
   if (theme->fwd_databaseID[0]!='\0') theme=ide_find_theme(handle,theme->database->databaseID,theme->number);

   glo_get_part(ident,1,wyraz1,'+');
   glo_get_part(ident,2,fleks1,'+');

   // pobieramy kolejne identyfikatory glowne tematu,
   ile1=glo_count_parts(handle,theme->identify,',');
   for (nr1=0;nr1<ile1;nr1++)
   {
      glo_get_part(theme->identify,nr1+1,temp1,',');
      glo_get_part(temp1,1,wyraz2,'+');
      glo_get_part(temp1,2,fleks2,'+');

      // porownujemy, czy podany identyfikator zawiera sie w podanym:
      // jesli sa rowne (wyrazy) to RETURN TRUE,
      if (strcmp(wyraz1,wyraz2)==0) return TRUE;

      // jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
      if (mode==MODE_EXACT) continue;

      switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
      {
         case FCMP_A_EQU_B:
         case FCMP_A_IN_B:
         case FCMP_A_PR_B: return TRUE;
      }
   }

   // pobieramy kolejne fakty proste z wszystkich baz faktow, ktore naleza do tego tematu,
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      for (fact=database->facts;fact!=NULL;fact=fact->next)
      {
         for (fs=fact->fs;fs!=NULL;fs=fs->next)
         {
            // sprawdzamy, czy fs nalezy do tematu
            if (ide_find_theme(handle,fs->databaseID,fs->theme_number)!=theme) continue;

            // porownujemy, czy podany identyfikator zawiera sie w pobranym fakcie prostym:
            if (exist_id_in_fs(handle,ident,fs,mode)==TRUE) return TRUE;
         }
      }
   }

   return FALSE;
}

// ***************************************************************************************
// EXTERN             Theme *ide_find_theme(handle,databaseID,theme_number)
// ***************************************************************************************
struct Theme *ide_find_theme(struct DeniseHandle* handle,char *databaseID,int theme_number)
{
   /*
   funkcja znajduje rzeczywisty (aktywny) temat, tzn. odszukuje temat
   theme_number w bazie databaseID, i jesli jest on przekierowany dalej, to
   odszukuje przekierowania, powtarzajac proces az znajdzie temat, ktory
   nie jest nigdzie dalej przekierowywany
   */

   struct FactDatabase *database;
   struct Theme *theme=NULL;
   char searchID[130];

//DeniseReport(handle,"!!!!!!!%s",databaseID);

   strcpy(searchID,databaseID);
/*
   if (handle==NULL)
	{
		DeniseReport(handle,"no HANDLE");
		return NULL;
	}

	if (handle->facts==NULL)
	{
		DeniseReport(handle,"no facts!");
		return NULL;
	}
*/
   START:
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      if (strcmp(database->databaseID,searchID)==0)
      {
         for (theme=database->themes;theme!=NULL;theme=theme->next)
         {
            if (theme->number==theme_number)
            {
               if (theme->fwd_databaseID[0]!='\0')
               {
                  strcpy(searchID,theme->fwd_databaseID);
                  theme_number=theme->fwd_number;
                  goto START;
               }
               else
               {
                  return theme;
               }
            }
         }
         return NULL;
      }
   }
   return NULL;
}

// ***************************************************************************************
// EXTERN                 int ide_free_theme_number(handle,database)
// ***************************************************************************************
int ide_free_theme_number(struct DeniseHandle* handle,struct FactDatabase *database)
{
   int nr1=1;

   if (database->themes==NULL) return nr1;

   for (;;nr1++)
   {
      if (ide_find_theme(handle,database->databaseID,nr1)==NULL) return nr1;
   }

   printf("a to gluuuupia nieeeeezapowiedzianka w ide_free_theme_number\n");
   return 0;
}

// ***************************************************************************************
// EXTERN                        ide_themes_list_free(head)
// ***************************************************************************************
void ide_themes_list_free(struct ThemesList *head)
{
   struct ThemesList *theme,*theme_next;

   if (head==NULL) return;

   for (theme=head;theme!=NULL;theme=theme_next)
   {
      theme_next=theme->next;
      free(theme);
   }
}

// ***************************************************************************************
// EXTERN                  int ide_accept_theme(handle,theme,ident)
// ***************************************************************************************
int ide_accept_theme(struct DeniseHandle* handle,struct Theme *theme,char *ident)
{
   /*
   funkcja sprawdza, czy kwantyfikator podany w ident obejmuje podany temat,

   algorytm:
   - pobieramy kolejne identyfikatory glowne i podrzedne tematu,
   - jesli ktorykolwiek jest zgodny z kwantyfikatorem, to zwracamy wynik,
   */

   struct FactDatabase *database;
   struct Fact *fact;
   struct FactSimple *fs;
   char temp1[100],wyraz1[100],fleks1[100],wyraz2[100],fleks2[100],*ptr;
   int ile1,nr1,result;

   // jesli temat jest forwardowany, to odszukujemy ostateczny
   if (theme->fwd_databaseID[0]!='\0') theme=ide_find_theme(handle,theme->database->databaseID,theme->number);

   glo_get_part(ident,1,wyraz1,'+');
   glo_get_part(ident,2,fleks1,'+');

   // pobieramy kolejne identyfikatory glowne tematu,
   ile1=glo_count_parts(handle,theme->identify,',');
   for (nr1=0;nr1<ile1;nr1++)
   {
      glo_get_part(theme->identify,nr1+1,temp1,',');
      glo_get_part(temp1,1,wyraz2,'+');
      glo_get_part(temp1,2,fleks2,'+');

      // porownujemy, czy podany identyfikator zawiera sie w podanym:
      // jesli sa rowne (wyrazy) to RETURN FCMP_A_EQU_B,
//      if (strcmp(wyraz1,wyraz2)==0) return FCMP_A_EQU_B;

      result=log_cmp_kwantyfikatory(handle,fleks1,fleks2);
      if (result!=FCMP_A_DIF_B) return result;
   }

   // pobieramy kolejne fakty proste z wszystkich baz faktow, ktore naleza do tego tematu,
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      for (fact=database->facts;fact!=NULL;fact=fact->next)
      {
         for (fs=fact->fs;fs!=NULL;fs=fs->next)
         {
            // sprawdzamy, czy fs nalezy do tematu
            if (ide_find_theme(handle,fs->databaseID,fs->theme_number)!=theme) continue;

            // pobieramy kolejne identyfikatory wyrazowe (glowne i podrzedne) faktu prostego,
            for (ptr=fs->sentence->id_subject,ile1=glo_count_parts(handle,ptr,','),nr1=0;nr1<ile1;nr1++)
            {
               glo_get_part(ptr,nr1+1,temp1,',');
               glo_get_part(temp1,1,wyraz2,'+');
               glo_get_part(temp1,2,fleks2,'+');

               // jesli to ostatni identyfikator glowny, to bierzemy teraz podrzedne
               if (nr1==ile1-1 && ptr==fs->sentence->id_subject && fs->sentence->id_global!=NULL && strlen(fs->sentence->id_global)>0)
               {
                  ptr=fs->sentence->id_global;
                  ile1=glo_count_parts(handle,ptr,',');
                  nr1=-1;
               }

               // porownujemy, czy podany identyfikator zawiera sie w podanym:
               // jesli sa rowne (wyrazy) to RETURN FCMP_A_EQU_B,
//               if (strcmp(wyraz1,wyraz2)==0) return FCMP_A_EQU_B;

               result=log_cmp_kwantyfikatory(handle,fleks1,fleks2);
               if (result!=FCMP_A_DIF_B) return result;
            }
         }
      }
   }

   return FCMP_A_DIF_B;
}

// ***************************************************************************************
// EXTERN                  int ide_cmp_themes(handle,theme1,theme2,mode)
// ***************************************************************************************
int ide_cmp_themes(struct DeniseHandle* handle,struct Theme *theme1,struct Theme *theme2,int mode)
{
   /*
   funkcja porownuje dwa tematy w podanym trybie.

   mode:
   MODE_EXACT - bez uwzgledniania znaczenia kwantyfikatorow,
   MODE_COMPATIBLE - z uwzglednianiem znaczenia kwantyfikatorow,

   funkcja zwraca wartosci:
   FCMP_A_DIF_B 2
   FCMP_A_EQU_B 3

   <w tej chwili zwraca tylko te dwie, ponizszych nie zwraca>
   FCMP_A_IN_B 4
   FCMP_B_IN_A 5
   FCMP_A_PR_B 6
   FCMP_B_PR_A 7
   FCMP_A_EQU_B_NEG 8
   FCMP_A_IN_B_NEG 9
   FCMP_B_IN_A_NEG 10
   FCMP_A_PR_B_NEG 11
   FCMP_B_PR_A_NEG 12

   warunki poprawnej identyfikacji:
   - temat 1 zawiera wszystkie identyfikatory tematu 2,
   - temat 2 zawiera wszystkie identyfikatory tematu 1,

   algorytm:
   */

   // jesli temat jest forwardowany, to odszukujemy ostateczny
   if (theme1->fwd_databaseID[0]!='\0') theme1=ide_find_theme(handle,theme1->database->databaseID,theme1->number);
   if (theme2->fwd_databaseID[0]!='\0') theme2=ide_find_theme(handle,theme2->database->databaseID,theme2->number);

   if (theme1==theme2 && theme1!=NULL) return FCMP_A_EQU_B;
   else if (theme1==NULL || theme2==NULL) return FCMP_A_DIF_B;

   if (theme2_in_theme1(handle,theme1,theme2,mode)==FALSE) return FCMP_A_DIF_B;
   if (theme2_in_theme1(handle,theme2,theme1,mode)==FALSE) return FCMP_A_DIF_B;

   return FCMP_A_EQU_B;
}

static BOOL theme2_in_theme1(struct DeniseHandle* handle,struct Theme *theme1,struct Theme *theme2,int mode)
{
   struct FactDatabase *database;
   struct Fact *fact;
   struct FactSimple *fs;
   char temp1[100];
   int ile,nr1;

   // pobieramy kolejne fakty proste z wszystkich baz faktow, ktore naleza do tego tematu,
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      for (fact=database->facts;fact!=NULL;fact=fact->next)
      {
         for (fs=fact->fs;fs!=NULL;fs=fs->next)
         {
            // sprawdzamy, czy fs nalezy do tematu
            if (ide_find_theme(handle,fs->databaseID,fs->theme_number)!=theme2) continue;

            //  pobieramy kolejne identyfikatory glowne faktu,
            for (ile=glo_count_parts(handle,fs->sentence->id_subject,','),nr1=0;nr1<ile;nr1++)
            {
               glo_get_part(fs->sentence->id_subject,nr1+1,temp1,',');

               // sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
               // (moga to byc identyfikatory glowne lub podrzedne),
               switch (exist_id_in_theme(handle,temp1[0]=='!' ? temp1+1 : temp1,theme1,mode))
               {
                  // jesli NIE:
                  case FALSE:
                     // jesli to identyfikator zanegowany to kontynuuj,
                     // jesli to identyfikator zwyczajny to FALSE,
                     if (temp1[0]!='!') return FALSE;
                     break;
                  // jesli TAK:
                  case TRUE:
                     // jesli to identyfikator zanegowany to return FALSE,
                     if (temp1[0]=='!') return FALSE;
                     // jesli to identyfikator zwyczajny to kontynuuj,
                     break;
               }
            }

            // pobieramy kolejne identyfikatory podrzedne faktu,
            // sprawdzamy, czy pobrany identyfikator zawiera sie w identyfikatorach tematu,
            // (moga to byc identyfikatory glowne lub podrzedne),
//            - jesli NIE:
//               - jesli to identyfikator zanegowany to kontynuuj,
//               - jesli to identyfikator zwyczajny to pobieramy nastepny temat,
//            - jesli TAK:
//               - jesli to identyfikator zanegowany to pobieramy nastepny temat,
//               - jesli to identyfikator zwyczajny to kontynuuj,
//            if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,"brak obslugi identyfikatorow podrzednych w fakcie");
         }
      }
   }

   return TRUE;
}
