// ***************************************************************************************
//                                     LOGIC - CONVERSION
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static int get_valueset(struct DeniseHandle *handle,char *logic,int ile_faktow,BOOL *values);
static void set_values(int *value,char *logic);
static BOOL boolean_result(char *logic);
static BOOL clause_1in2(BOOL *val1,BOOL *val2,int ile_odp);
static void simplification_then(char *src,char *dst);
static void simplification_xor(char *src,char *dst);
static int count_logic_facts(char *wyrazenie);

// ***************************************************************************************
// EXTERN                       BOOL log_islogic(char *src)
// ***************************************************************************************
BOOL log_islogic(struct DeniseHandle *handle,char *src)
{
   /*
   funkcja sprawdza, czy podany tekst jest formalnym zapisem logicznym
   poprawny zapis logiczny: A or B and C or not D and (E or F)
   */

   char temp[500];
   int nr1,ile,test;

   if (strlen(src)==0) return FALSE;
   ile=glo_count_parts(handle,src,' ');
   if (ile==0) return FALSE;

   for (nr1=0;nr1<ile;nr1++)
   {
      if (glo_get_part(src,nr1+1,temp,' ')==0)
      {
         printf("blad: 5\n");
         return FALSE;
      }

      test=0;
      for (;;)
      {
         if (temp[0]=='(')
         {
            test=1;
            memmove(temp,temp+1,strlen(temp));
         }
         if (temp[strlen(temp)-1]==')')
         {
            test=1;
            temp[strlen(temp)-1]='\0';
         }

         if (temp[0]!='(' && temp[strlen(temp)-1]!=')') break;
      }
      if (test==1)
      {
         if (log_islogic(handle,temp)==FALSE) return FALSE;
         else continue;
      }

      if (strcmp(temp,"and")==0 ||
          strcmp(temp,"or")==0 ||
          strcmp(temp,"not")==0 ||
          strcmp(temp,"xor")==0 ||
          strcmp(temp,"then")==0 ||
          (strlen(temp)==1 && temp[0]>='A' && temp[0]<='Z')) continue;
      else return FALSE;
   }
   return TRUE;
}

// ***************************************************************************************
// EXTERN                 void log_descr2expr(char *src,char *dst)
// ***************************************************************************************
void log_descr2expr(struct DeniseHandle *handle,char *src,char *dst)
{
   /*
   funkcja zamienia opis slowny logiczny na postac rozumiana przez program
   A or B = A3B
   */

   char temp[1000],res[1000],temp1[1000];
   int nr1,nr2,nr3,ile,ile_nawiasow;

   dst[0]='\0';

   nr2=0;
   if (strlen(src)==0) return;
   ile=glo_count_parts(handle,src,' ');
   for (nr1=0;nr1<ile;nr1++)
   {
      ile_nawiasow=0;
      if (glo_get_part(src,nr1+1,temp,' ')==0)
      {
         printf("blad: 6\n");
         return;
      }

      for (ile_nawiasow=0;ile_nawiasow<(int) strlen(temp);ile_nawiasow++)
         if (temp[ile_nawiasow]!='(') break;

      if (ile_nawiasow>0)
      {
         memmove(temp,temp+ile_nawiasow,(int) strlen(temp)-ile_nawiasow+1);
         for (;ile_nawiasow!=0;ile_nawiasow--)
            res[nr2++]='(';
      }

      for (ile_nawiasow=0;ile_nawiasow<(int) strlen(temp);ile_nawiasow++)
         if (temp[strlen(temp)-1-ile_nawiasow]!=')') break;

      if (ile_nawiasow>0) temp[strlen(temp)-ile_nawiasow]='\0';

      if (strchr(temp,' ')!=NULL)
      {
         log_descr2expr(handle,temp,temp1);
         for (nr3=0;nr3<(int) strlen(temp1);nr3++) res[nr2++]=temp1[nr3];
      }
      else
      {
         if (strcmp(temp,"and")==0) res[nr2++]=CON_AND+'1'-1;
         if (strcmp(temp,"or")==0) res[nr2++]=CON_OR+'1'-1;
         if (strcmp(temp,"not")==0) res[nr2++]=CON_NOT+'1'-1;
         if (strcmp(temp,"xor")==0) res[nr2++]=CON_XOR+'1'-1;
         if (strcmp(temp,"then")==0) res[nr2++]=CON_THEN+'1'-1;
         if (strlen(temp)==1 && temp[0]>='A' && temp[0]<='Z') res[nr2++]=temp[0];
      }

      if (ile_nawiasow>0)
      {
         for (;ile_nawiasow>0;ile_nawiasow--)
            res[nr2++]=')';
      }
   }
   res[nr2]='\0';
   strcpy(dst,res);
}

// ***************************************************************************************
// EXTERN      void log_expr2clause(DeniseHandle *handle,char *src,char *dst)
// ***************************************************************************************
void log_expr2clause(struct DeniseHandle *handle,char *src,char *dst)
{
   /*
   funkcja przeksztalca wyrazenie na forme klauzul (klauzule sa rozdzielone przecinkiem)
	funkcja jest uniwersalna

	zasada dzialania:
	<etap 1>
	- generujemy tablice wartosci podanego wyrazenia dla
	  wszystkich mozliwych kombinacji wartosci parametrow,
     zapamietujac w niej wynik: TRUE, FALSE
	<etap 2>
	- program generuje ciag poczatkowy o wartosci 0 na kazdym polu,
	  przy czym liczba pol jest rowna liczbie zmiennych w wyrazeniu,
	- nastepnie rozpoczyna sie karuzela - licznik trojkowy,
	  gdzie kazda pozycja przyjmuje wartosci 0,1,2
		- dla danego zestawu wartosci generowany jest opis logiczny:
		  0 - brak elementu
		  1 - element
		  2 - element zanegowany
		- sprawdzane jest, czy ten opis WYELIMINUJE (jest sprzeczny) z opisem
		  podanym w src, czyli - czy dla jakiejs wartosci parametrow opis src
		  przyjmuje wartosc 1, a opis generowany 0; jesli TAK, to pomijamy
		  ten generowany opis, a w przeciwnym razie zapamietujemy go,
	<etap 3>
	optymalizacja klauzul
	- pobieramy klauzule A,
		- obliczamy wartosc,
		- pobieramy klauzule B,
			- obliczamy wartosc,
			- jesli klauzula A miesci sie w B, to kasujemy A i nast. A,
			- jesli klauzula B miesci sie w A, to kasujemy B i nast. B,

A or B and not C
=>
A or B,A or B or C,A or not C,A or B or not C,A or not B or not C
A or B,			   A or not C

A or B,not A or not B,not C

			  A3B	  A3B3C	  A31C	  A3B31C  A31B31C
a b c ALL	1 czlon	2 czlon	3 czlon	4 czlon	5 czlon
0 0 0 00	0		0		1		1		1
0 0 1 00	0		1		0		0		1
0 1 0 11	1		1		1		1		1
0 1 1 00	1		1		0		1		0
1 0 0 11	1		1		1		1		1
1 0 1 11	1		1		1		1		1
1 1 0 11	1		1		1		1		1
1 1 1 11	1		1		1		1		1


0 0 0 = 0
1 0 0 = 1
0 1 0 = 1
1 1 0 = 1
0 0 1 = 0
1 0 1 = 1
0 1 1 = 0
1 1 1 = 1

   */

	BOOL values[1000]; // tablica wynikow podanego wyrazenia dla kolejnych zestawow wartosci
	BOOL values1[100],values2[100];
	int gen[100]; // parametry generowanego opisu: 0/1/2
	char temp1[1000],temp2[1000],gen_logic[1000],con_or[10],dst_temp[10000];
	int nr1,nr2,ile_faktow,valueset;

	dst[0]='\0';
   dst_temp[0]='\0';

	sprintf(con_or,"%d",CON_OR);

	// ile roznych faktow?
	ile_faktow=0;
	for (nr1=0;nr1<(int) strlen(src);nr1++)
		if (src[nr1]>='A' && src[nr1]<='Z') ile_faktow=max(ile_faktow,src[nr1]-'A'+1);

//DeniseReport(handle,"BEGIN NEW expr2clause: source: %s",src);

	// generujemy tablice wartosci podanego wyrazenia dla
	// wszystkich mozliwych kombinacji wartosci parametrow,
   // zapamietujac w niej wynik: TRUE, FALSE
	valueset=get_valueset(handle,src,ile_faktow,values);

//DeniseReport(handle,"values: %d",valueset);

	// program generuje ciag poczatkowy o wartosci 0 na kazdym polu,
	// przy czym liczba pol jest rowna liczbie zmiennych w wyrazeniu,
	for (nr1=0;nr1<ile_faktow;nr1++) gen[nr1]=0;

//DeniseReport(handle,"faktow: %d",ile_faktow);

	// nastepnie rozpoczyna sie karuzela - licznik trojkowy,
	// gdzie kazda pozycja przyjmuje wartosci 0,1,2
	// probujemy zmienic wartosc na najbardziej lewej pozycji
	for (nr1=0;nr1<ile_faktow;nr1++)
	{
		if (gen[nr1]<2)
		{
			gen[nr1]++; // 1, 2
			// wszystkie wczesniejsze pozycje zerujemy
			nr1--;
			for (;nr1>=0;nr1--) gen[nr1]=0;

			// dla danego zestawu wartosci generowany jest opis logiczny:
	  		// 0 - brak elementu
	  		// 1 - element
	  		// 2 - element zanegowany
			gen_logic[0]='\0';
			for (nr2=0;nr2<ile_faktow;nr2++)
			{
				if (gen_logic[0]!='\0' && gen[nr2]!=0) strcat(gen_logic,con_or);
				if (gen[nr2]==1)
				{
					sprintf(temp1,"%c",'A'+nr2);
					strcat(gen_logic,temp1);
				}
				else if (gen[nr2]==2)
				{
					sprintf(temp1,"%d%c",CON_NOT,'A'+nr2);
					strcat(gen_logic,temp1);
				}
			}

//DeniseReport(handle,"GEN: %s",gen_logic);

			// sprawdzane jest, czy ten opis WYELIMINUJE (jest sprzeczny) z opisem
	  		// podanym w src, czyli - czy dla jakiejs wartosci parametrow opis src
	  		// przyjmuje wartosc 1, a opis generowany 0; jesli TAK, to pomijamy
	  		// ten generowany opis, a w przeciwnym razie zapamietujemy go,
			if (get_valueset(handle,gen_logic,ile_faktow,values1)!=valueset) DeniseReport(handle,"logic2 spok error");
			for (nr2=0;nr2<valueset;nr2++)
				if (values[nr2]==TRUE && values1[nr2]==FALSE) goto NEXT_OPIS;

//DeniseReport(handle,"ACCEPTED GEN: %s",gen_logic);
//printf("ACCEPTED:\n");
//for (nr2=0;nr2<valueset;nr2++)
//	DeniseReport(handle,"%d = %d",values[nr2]==TRUE ? 1 : 0,values1[nr2]==TRUE ? 1 : 0);

			// zapamietujemy ten opis
			if (dst_temp[0]!='\0')
			{
				sprintf(temp1,",");
				strcat(dst_temp,temp1);
			}
			strcat(dst_temp,gen_logic);

			NEXT_OPIS:;
			nr1=-1;
		}
	}

//DeniseReport(handle,"RESULT: !%s!",dst_temp);
	if (dst_temp[0]=='\0')
	{
		dst[0]='\0';
		return;
	}
//return;
	// optymalizacja klauzul
	// pobieramy klauzule A (nr1),
	for (nr1=0;nr1<glo_count_parts(handle,dst_temp,',');nr1++)
	{
		// obliczamy wartosc,
		glo_get_part(dst_temp,nr1+1,temp1,',');
		valueset=get_valueset(handle,temp1,ile_faktow,values1);
		// pobieramy klauzule B,
		for (nr2=nr1+1;nr2<glo_count_parts(handle,dst_temp,',');nr2++)
		{
			// obliczamy wartosc,
			glo_get_part(dst_temp,nr2+1,temp2,',');
			if (get_valueset(handle,temp2,ile_faktow,values2)!=valueset) DeniseReport(handle,"biedronka spadla z dachu... ;( ERROR!");

			// jesli klauzula A miesci sie w B, to kasujemy A i nast. A,
			if (clause_1in2(values1,values2,valueset)==TRUE)
			{
				glo_delete_part_sep(handle,dst_temp,nr1+1,',');
//printf("nr1 jest w nr2: %s w %s\n",temp1,temp2);
				nr1=max(nr1-1,-1);
				break;
			}
			// jesli klauzula B miesci sie w A, to kasujemy B i nast. B,
			else if (clause_1in2(values2,values1,valueset)==TRUE)
			{
//printf("nr2 jest w nr1: %s w %s\n",temp2,temp1);
				glo_delete_part_sep(handle,dst_temp,nr2+1,',');
				nr2=max(nr2-1,nr1);
				continue;
			}
		}
	}

	strcpy(dst,dst_temp);
//DeniseReport(handle,"END expr2clause: dst '%s'",dst);
//printf("UPR!!!: !%s!\n",dst);

}

static int get_valueset(struct DeniseHandle *handle,char *logic,int ile_faktow,BOOL *values)
{
	/*
	funkcja zwraca w tablicy wyniki danego wyrazenia logicznego o danej liczbie faktow
	zwraca liczbe wartosci
	*/

	char temp[1000];
	int nr1,valueset,value[100];

//DeniseReport(handle,"LOGIC: %s",logic);
//DeniseReport(handle,"a b c  res");

	// generujemy tablice wartosci podanego wyrazenia dla
	// wszystkich mozliwych kombinacji wartosci parametrow,
   // zapamietujac w niej wynik: TRUE, FALSE
	for (nr1=0;nr1<ile_faktow;nr1++) value[nr1]=0;
	// probujemy zmienic wartosc na najbardziej lewej pozycji
	valueset=0;
	strcpy(temp,logic);
	set_values(value,temp);
	values[valueset++]=boolean_result(temp);
//DeniseReport(handle,"%d %d %d = %d",value[0],value[1],value[2],values[valueset-1]==TRUE ? 1 : 0);

	for (nr1=0;nr1<ile_faktow;nr1++)
	{
		if (value[nr1]==0)
		{
			value[nr1]=1;
			// wszystkie wczesniejsze pozycje zerujemy
			nr1--;
			for (;nr1>=0;nr1--) value[nr1]=0;

			strcpy(temp,logic);
			set_values(value,temp);
			values[valueset++]=boolean_result(temp);
//DeniseReport(handle,"%d %d %d = %d",value[0],value[1],value[2],values[valueset-1]==TRUE ? 1 : 0);
			nr1=-1;
		}
	}

	return valueset;
}

static BOOL clause_1in2(BOOL *val1,BOOL *val2,int ile_odp)
{
	/*
	klauzula 1 zawiera sie w klauzuli 2, jesli (powownujemy zestawy odpowiedzi)
	- w klauzuli 2 wystepuja wszystkie zera klauzuli 1,
	  (klauzula 1 nie ma zer, ktore nie wystepuja w 2),

	funkcja zwraca TRUE, jesli klauzula 1 zawiera sie w klauzuli 2,
	*/

	int nr1;

	for (nr1=0;nr1<ile_odp;nr1++)
	{
		// w klauzuli 2 wystepuja wszystkie zera klauzuli 1,
		if (val1[nr1]==FALSE && val2[nr1]!=FALSE) return FALSE;
	}

	return TRUE;
}

static void set_values(int *value,char *logic)
{
	// funkcja wstawia w miejsca zmiennych odpowiednie wartosci logiczne: 0/1
	// a w miejsce operatorow: 1->A, 2->B, 3->C
	int nr1;
	for (nr1=0;nr1<(int) strlen(logic);nr1++)
	{
		if (logic[nr1]>='A' && logic[nr1]<='Z') logic[nr1]=value[logic[nr1]-'A']==0 ? '0' : '1';
		else if (logic[nr1]!='(' && logic[nr1]!=')') logic[nr1]=logic[nr1]-'1'+'A';
	}
}

static BOOL boolean_result(char *logic)
{
	/*
	funkcja oblicza dane wyrazenie logiczne

	algorytm:
	kolejnosc jest wazna
	gdzie: NOT = A, AND = B, OR = C
	- (0) = 0
	- (1) = 1
	- not 0 = 1
	- not 1 = 0
	- 0 and 0 = 0
	- 0 and 1 = 0
	- 1 and 0 = 0
	- 1 and 1 = 1
	- 0 or 0 = 0
	- 0 or 1 = 1
	- 1 or 0 = 1
	- 1 or 1 = 1
	*/	

	int nr1;
	char *ptr,temp[1000];
	struct
	{
		char *opis;
		char result;
	} math[12]={{"(0)",'0'},
					{"(1)",'1'},
					{"A0",'1'},
					{"A1",'0'},
					{"0B0",'0'},
					{"0B1",'0'},
					{"1B0",'0'},
					{"1B1",'1'},
					{"0C0",'0'},
					{"0C1",'1'},
					{"1C0",'1'},
					{"1C1",'1'}};

//printf("OPIS: !%s!\n",logic);

	if (strlen(logic)==0)
	{
		printf("blad! LOGIC = '' w BOOLEAN RESULT logic2\n");
		return FALSE;
	}

	strcpy(temp,logic);
	START:
	for (nr1=0;nr1<12;nr1++)
	{
		ptr=strstr(logic,math[nr1].opis);
		if (ptr!=NULL)
		{
			ptr[0]=math[nr1].result;
			memmove(ptr+1,ptr+strlen(math[nr1].opis),strlen(ptr+strlen(math[nr1].opis))+1);
			goto START;
		}
	}

	if (strlen(logic)!=1)
	{
		printf("LOGIC2 - BOOLEAN RESULT - ERRROR: '%s'->'%s'\n",temp,logic);
		return FALSE;
	}

	return logic[0]=='0' ? FALSE : TRUE;
}

// ***************************************************************************************
// EXTERN              void log_expr_simplification(char *src,char *dst)
// ***************************************************************************************
int log_expr_simplification(char *src,char *dst)
{
   /*
	zadaniem funkcji jest zamiana wszystkich zlozonych operatorow na odpowiednie
	wyrazenia korzystajace tylko z operatorow AND, OR, NOT

	konwersja (najwyzszy priorytet ma then)
	1. A then B == not (A) or (B) <- dwie klauzule (nawiasy sa domyslne!!!)
	2. A xor B (xor C xor D...) == (A or B) and (not A or not B) <- dwie klauzule (nawiasy sa domyslne!!!)
   */

	char temp[1000];
	int nr1,ile;

	// wazna kolejnosc! UWAGA! nie obsluguje poprawnie wiecej niz 1 XOR i 1 THEN!!!
	// bo po co?
	for (ile=0,nr1=0;nr1<(int) strlen(src);nr1++) if (src[nr1]-'1'+1==CON_XOR) ile++;
	if (ile>1)
	{
		strcpy(dst,src);
		return -1;
	}
	for (ile=0,nr1=0;nr1<(int) strlen(src);nr1++) if (src[nr1]-'1'+1==CON_THEN) ile++;
	if (ile>1)
	{
		strcpy(dst,src);
		return -1;
	}

	simplification_then(src,temp);
	simplification_xor(temp,dst);

	return 0;
}

static void simplification_then(char *src,char *dst)
{
	/*
	konwersja THEN

	A then B == not (A) or (B) <- dwie klauzule (nawiasy sa domyslne!!!)

	algorytm:
	- tworzymy grupy A, B faktow polaczonych AND i OR i XOR
	  A, B - grupy polaczone AND'ami, OR'ami, XOR'ami oraz NOT'ami
	- pobieramy do tablicy kolejne sekcje rozdzielone THEN i opatrujemy nawiasami,
	- zapisujemy w nowy sposob:
		- zmieniamy numer od pierwszej do ostatniej sekcji:
			- jest numer wiekszy niz 1, to zapisujemy operator OR,
			- zapisujemy kolejne sekcje polaczone operatorem AND i NOT,
			  pamietajac, aby sekcja o danym numerze nie miala NOTa,
	*/

	int nr1,nr2,nr3,ile_grup,poz;
	char temp[20][500],src1[500],con_or[10],con_not[10],con_and[10],con_xor[10];

	sprintf(con_and,"%d",CON_AND);
	sprintf(con_or,"%d",CON_OR);
	sprintf(con_not,"%d",CON_NOT);
	sprintf(con_xor,"%d",CON_XOR);

	strcpy(src1,src);
	START:;
	// kasowanie zbednych nawiasow zewnetrznych wyrazenia
	if (src1[0]=='(')
	{
		nr2=0;
		for (nr1=0;nr1<(int) strlen(src1)-1;nr1++)
		{
			if (src1[nr1]=='(') nr2++;
			else if (src1[nr1]==')') nr2--;
			if (nr2==0) goto END_DEL;
		}
		src1[strlen(src1)-1]='\0';
		memmove(src1,src1+1,strlen(src1)+1);
		goto START;

		END_DEL:;
	}

//printf("src: %s\n",src1);

   // KONWERSJA THEN
   // tworzymy tablice grup - elementw rozdzielonych THENami
	poz=0;
	ile_grup=0;
   temp[ile_grup][poz++]='(';
   for (nr1=0;nr1<(int) strlen(src1);nr1++)
   {
      // spacje pomijamy
      if (src1[nr1]==' ') continue;

      // fakty (A,B,C...) dopisujemy, operatory NOT/AND/OR tez dopisujemy
      if ((src1[nr1]>='A' && src1[nr1]<='Z') ||
		  src1[nr1]-'1'+1==CON_NOT ||
		  src1[nr1]-'1'+1==CON_AND ||
		  src1[nr1]-'1'+1==CON_OR ||
		  src1[nr1]-'1'+1==CON_XOR ||
		  src1[nr1]=='(' || src1[nr1]==')') {temp[ile_grup][poz++]=src1[nr1]; continue;}

      // w przeciwnym razie nowa grupa
      temp[ile_grup][poz++]=')';
      temp[ile_grup++][poz]='\0'; // konczymy grupe

		// kasujemy zewnetrzne nawiasy jest nie sa potrzebne
		START1:;
		if (temp[ile_grup-1][0]=='(')
		{
			nr2=0;
			for (nr3=0;nr3<(int) strlen(temp[ile_grup-1])-1;nr3++)
			{
				if (temp[ile_grup-1][nr3]=='(') nr2++;
				else if (temp[ile_grup-1][nr3]==')') nr2--;
				if (nr2==0) goto END_DEL1;
			}
			temp[ile_grup-1][strlen(temp[ile_grup-1])-1]='\0';
			memmove(temp[ile_grup-1],temp[ile_grup-1]+1,strlen(temp[ile_grup-1])+1);
			goto START1;
			END_DEL1:;
		}

		// dalej...
      poz=0;
	   temp[ile_grup][poz++]='(';
   }
   temp[ile_grup][poz++]=')';
   temp[ile_grup][poz]='\0'; // konczymy grupe
	ile_grup++;

	// kasujemy zewnetrzne nawiasy jest nie sa potrzebne
	START2:;
	if (temp[ile_grup-1][0]=='(')
	{
		nr2=0;
		for (nr3=0;nr3<(int) strlen(temp[ile_grup-1])-1;nr3++)
		{
			if (temp[ile_grup-1][nr3]=='(') nr2++;
			else if (temp[ile_grup-1][nr3]==')') nr2--;
			if (nr2==0) goto END_DEL2;
		}
		temp[ile_grup-1][strlen(temp[ile_grup-1])-1]='\0';
		memmove(temp[ile_grup-1],temp[ile_grup-1]+1,strlen(temp[ile_grup-1])+1);
		goto START2;
		END_DEL2:;
	}

	// pobieramy do tablicy kolejne sekcje rozdzielone THEN i opatrujemy nawiasami,
	dst[0]='\0';
	// zapisujemy w nowy sposob:
	// zmieniamy numer od pierwszej do ostatniej sekcji:
	for (nr1=0;nr1<ile_grup;nr1++)
	{
		if (nr1==0 && ile_grup>1) strcat(dst,con_not);
		else if (nr1>0) strcat(dst,con_or);
		strcat(dst,temp[nr1]);
	}
}

static void simplification_xor(char *src,char *dst)
{
	/*
	konwersja XORow

	A xor B (xor C xor D...) == (A or B) and (not A or not B) <- dwie klauzule (nawiasy sa domyslne!!!)

	algorytm:
	- tworzymy grupy A, B faktow poczonych AND i OR
	  A, B - grupy polaczone AND'ami, OR'ami oraz NOT'ami
	- pobieramy do tablicy kolejne sekcje rozdzielone XOR i opatrujemy nawiasami,
	- zapisujemy w nowy sposob:
		- zmieniamy numer od pierwszej do ostatniej sekcji:
			- jest numer wiekszy niz 1, to zapisujemy operator OR,
			- zapisujemy kolejne sekcje polaczone operatorem AND i NOT,
			  pamietajac, aby sekcja o danym numerze nie miala NOTa,
	*/
	 
	int nr1,nr2,nr3,ile_grup,poz;
	char temp[20][500],src1[500],con_or[10],con_not[10],con_and[10];

	sprintf(con_and,"%d",CON_AND);
	sprintf(con_or,"%d",CON_OR);
	sprintf(con_not,"%d",CON_NOT);

	strcpy(src1,src);
	START:;
	// kasowanie zbednych nawiasow zewnetrznych wyrazenia
	if (src1[0]=='(')
	{
		nr2=0;
		for (nr1=0;nr1<(int) strlen(src1)-1;nr1++)
		{
			if (src1[nr1]=='(') nr2++;
			else if (src1[nr1]==')') nr2--;
			if (nr2==0) goto END_DEL;
		}
		src1[strlen(src1)-1]='\0';
		memmove(src1,src1+1,strlen(src1)+1);
		goto START;

		END_DEL:;
	}

//printf("src: %s\n",src1);

   // KONWERSJA XOR
   // tworzymy tablice grup - elementw rozdzielonych XORami
	poz=0;
	ile_grup=0;
   temp[ile_grup][poz++]='(';
   for (nr1=0;nr1<(int) strlen(src1);nr1++)
   {
      // spacje pomijamy
      if (src1[nr1]==' ') continue;

      // fakty (A,B,C...) dopisujemy, operatory NOT/AND/OR tez dopisujemy
      if ((src1[nr1]>='A' && src1[nr1]<='Z') ||
		  src1[nr1]-'1'+1==CON_NOT || 
		  src1[nr1]-'1'+1==CON_AND || 
		  src1[nr1]-'1'+1==CON_OR || 
		  src1[nr1]-'1'+1==CON_THEN || 
		  src1[nr1]=='(' || src1[nr1]==')') {temp[ile_grup][poz++]=src1[nr1]; continue;}

      // w przeciwnym razie nowa grupa
      temp[ile_grup][poz++]=')';
      temp[ile_grup++][poz]='\0'; // konczymy grupe

		// kasujemy zewnetrzne nawiasy jest nie sa potrzebne
		START1:;
		if (temp[ile_grup-1][0]=='(')
		{
			nr2=0;
			for (nr3=0;nr3<(int) strlen(temp[ile_grup-1])-1;nr3++)
			{
				if (temp[ile_grup-1][nr3]=='(') nr2++;
				else if (temp[ile_grup-1][nr3]==')') nr2--;
				if (nr2==0) goto END_DEL1;
			}
			temp[ile_grup-1][strlen(temp[ile_grup-1])-1]='\0';
			memmove(temp[ile_grup-1],temp[ile_grup-1]+1,strlen(temp[ile_grup-1])+1);
			goto START1;
			END_DEL1:;
		}

		// dalej...
      poz=0;
	   temp[ile_grup][poz++]='(';
   }
   temp[ile_grup][poz++]=')';
   temp[ile_grup][poz]='\0'; // konczymy grupe
	ile_grup++;

	// kasujemy zewnetrzne nawiasy jest nie sa potrzebne
	START2:;
	if (temp[ile_grup-1][0]=='(')
	{
		nr2=0;
		for (nr3=0;nr3<(int) strlen(temp[ile_grup-1])-1;nr3++)
		{
			if (temp[ile_grup-1][nr3]=='(') nr2++;
			else if (temp[ile_grup-1][nr3]==')') nr2--;
			if (nr2==0) goto END_DEL2;
		}
		temp[ile_grup-1][strlen(temp[ile_grup-1])-1]='\0';
		memmove(temp[ile_grup-1],temp[ile_grup-1]+1,strlen(temp[ile_grup-1])+1);
		goto START2;
		END_DEL2:;
	}

	// pobieramy do tablicy kolejne sekcje rozdzielone XOR i opatrujemy nawiasami,
	dst[0]='\0';
	// zapisujemy w nowy sposob:
	// zmieniamy numer od pierwszej do ostatniej sekcji:
	for (nr1=0;nr1<ile_grup;nr1++)
	{
		// jest numer wiekszy niz 1, to zapisujemy operator OR,
		if (nr1>0) strcat(dst,con_or);
		
		// zapisujemy kolejne sekcje polaczone operatorem AND i NOT,
		// pamietajac, aby sekcja o danym numerze nie miala NOTa,
		for (nr2=0;nr2<ile_grup;nr2++)
		{
			if (nr2!=0) strcat(dst,con_and);
			if (nr2!=nr1) strcat(dst,con_not);
			strcat(dst,temp[nr2]);
		}
	}

//DeniseReport(handle,"!%s! (ile grup: %d)\n",dst,ile_grup);
}

// ***************************************************************************************
// EXTERN               void log_expr_description(DeniseHandle *,char *src,char *dst)
// ***************************************************************************************
void log_expr_description(struct DeniseHandle *handle,char *src,char *dst)
{
   char res[1000],temp[1000];
   int nr1;

	if (src==NULL || src[0]=='\0')
	{
		sprintf(dst,UNN_GetLocaleString(handle,MSG_EMPTY_CLAUSE_TRUE,"wynikiem jest klauzula pusta (logicznie zawsze prawda)"));
		return;
	}
   
   dst[0]='\0';

   res[0]='\0';
   for (nr1=0;nr1<(int) strlen(src);nr1++)
   {
      switch (src[nr1]-'1'+1)
      {
         case CON_NOT: sprintf(temp,"not "); break;
         case CON_AND: sprintf(temp," and "); break;
         case CON_OR: sprintf(temp," or "); break;
         case CON_XOR: sprintf(temp," xor "); break;
         case CON_THEN: sprintf(temp," then "); break;
         default: sprintf(temp,"%c",src[nr1]); break;
      }
      strcat(res,temp);
   }

   strcpy(dst,res);
}

// ***************************************************************************************
// EXTERN       int log_fact_simplification(DeniseHandle *handle,Fact *fact)
// ***************************************************************************************
int log_fact_simplification(struct DeniseHandle *handle,struct Fact *fact)
{
	/*
   funkcja sprawdza, czy mozna uproscic fakt, i dokonuje tego, jesli tak
	chodzi o wyeliminowanie tych samych lub sprzecznych faktow prostych

   funkcja zwraca:
   - -1 jesli wykryto wewnetrzna sprzecznosc,
   - +1 jesli tautologia - zawsze prawdziwe (not A or A)
   -  0 w przeciwnym razie,

   algorytm:
   - pobieramy kolejne fakty proste A,
   	- pobieramy kolejne fakty proste nastepne B,
      	- jesli fakty sa zgodne lub sprzeczne, to zapisujemy to w tablicy,
           i kasujemy fakt prosty B,
   	- uaktualniamy zapis w logic,
	- optymalizujemy sam zapis logiczny
	*/

   struct FactSimple *fsA,*fsB,*fsB_last;
   int res,nr1,nr2,nr3,ile_faktow;
   char con_and[2];
   char logic[100],temp[1000],temp1[100],*ptr,x_copy[100];
	struct
	{
		char *opis;
		char result; // 
        int poz; // od 0
	} table[9]={{"(x)",'0',1},
			   	{"1x21x",'1',1},
				{"1x2x",'!',-1}, // sprzecznosc
				{"x21x",'!',-1}, // sprzecznosc
				{"x2x",'0',0},
				{"1x31x",'1',1},
				{"1x3x",'0',-1}, // nic
				{"x31x",'0',-1}, // nic
				{"x3x",'0',0}};

	sprintf(con_and,"%d",CON_AND);

   if (fact->fs==NULL) return -1;

	// tworzymy jeden ciag opisu logicznego i rzucamy go funkcji rekurencyjnej na pozarcie
	logic[0]='\0';
	for (nr1=0;nr1<fact->ile_klauzul;nr1++)
	{
		if (nr1>0) strcat(logic,con_and);
//		strcat(logic,"(");

		// nie optymalizujemy klauzul z OR jesli jest wiecej niz 2 klauzule
		if (fact->ile_klauzul>1)
		{
			for (nr2=0;nr2<(int) strlen(fact->logic[nr1]);nr2++)
			{
				if (fact->logic[nr1][nr2]-'1'+1==CON_OR) return 0;
			}
		}
		
		strcat(logic,fact->logic[nr1]);
//		strcat(logic,")");
	}

//DeniseReport(handle,"logikaA: '%s'",logic);

//ZLE: marek ma albo nie ma?
//nie identyfikuje!

//czy to sprzecznosc: marek ma albo ma? BEZ SENSU!!!

   // pobieramy kolejne fakty proste A,
   for (fsA=fact->fs,nr1=0;fsA->next!=NULL;fsA=fsA->next,nr1++)
   {
//DeniseReport(handle,"themenumber: %d",fsA->theme_number);
//continue;
   	// pobieramy kolejne fakty proste nastepne B,
		RESTART:;
		if (fsA->next==NULL) break;
   	for (fsB=fsA->next,nr2=nr1+1;fsB!=NULL;fsB_last=fsB,fsB=fsB->next,nr2++)
      {
			// jesli fakty sa zgodne lub sprzeczne, to zapisujemy to w tablicy,
         // UWAGA! kontrola nie obejmuje roznych form: A_IN_B, A_PR_B itd.
         // jest wiec troche prostsza niz rachunek logiczny stosowany we wnioskowaniu
			/*
	        	FCMP_A_DIF_B 2
			   FCMP_A_EQU_B 3
			   FCMP_A_IN_B 4
			   FCMP_B_IN_A 5
			   FCMP_A_PR_B 6
			   FCMP_B_PR_A 7
			   FCMP_A_EQU_B_NEG 8
			   FCMP_A_IN_B_NEG 9
			   FCMP_B_IN_A_NEG 10
			   FCMP_A_PR_B_NEG 11
			   FCMP_B_PR_A_NEG 12
			*/
			temp[0]='\0';
			res=log_fscmp(handle,fsA,fsB,CMP_ALL_GROUPS);
			if (res==FCMP_A_EQU_B) sprintf(temp,"%c",'A'+nr1);
			else if (res==FCMP_A_EQU_B_NEG) sprintf(temp,"%d%c",CON_NOT,'A'+nr1);

//DeniseReport(handle,"res: %d",res);

         if (temp[0]=='\0') continue;
   		// uaktualniamy zapis w logic,
         temp1[0]='\0';
         for (nr3=0;nr3<(int) strlen(logic);nr3++)
         {
         	if (logic[nr3]=='A'+nr2) strcat(temp1,temp);
            else if (logic[nr3]>'A'+nr2)
            {
					temp1[strlen(temp1)+1]='\0';
               temp1[strlen(temp1)]=logic[nr3]-1;
            }
            else
            {
					temp1[strlen(temp1)+1]='\0';
               temp1[strlen(temp1)]=logic[nr3];
            }
         }
			strcpy(logic,temp1);

         //  i kasujemy fakt prosty B,
			if (fsB==fsA->next) fsA->next=fsB->next;
         else fsB_last->next=fsB->next;
			dat_fact_simple_free(fsB);
			goto RESTART;
		}
   }

   // optymalizujemy sam zapis logiczny
	/*
	algorytm:
	kolejnosc jest wazna
	- (x) = x
	- 1x21x = 1x [not x and not x = not x]
	- 1x2x = SPRZECZNOSC [not x and x = SPRZECZNOSC]
	- x21x = SPRZECZNOSC [x and not x = SPRZECZNOSC]
	- x2x = x [x and x = x]
	- 1x31x = 1x [not x or not x = not x]
    - 1x3x = PRAWDA [not x or x = PRAWDA]
	- x31x = PRAWDA [x or not x = PRAWDA]
	- x3x = x [x or x = x]
	*/
/*
	struct
	{
		char *opis;
		char result;
      int poz;
	} table[9]={"(x)",'0',1,
			   	"1x21x",'1',1,
					"1x2x",'!',0, // sprzecznosc
					"x21x",'!',0, // sprzecznosc
					"x2x",'0',0,
					"1x31x",'1',1,
               "1x3x",'0',-1,
               "x31x",'0',-1,
               "x3x",'0',0};
*/

//DeniseReport(handle,"logikaB: '%s'",logic);

	ile_faktow=0;
	for (nr1=0;nr1<(int) strlen(logic);nr1++)
	{
		if (logic[nr1]>='A' && logic[nr1]<='Z') ile_faktow=max(ile_faktow,logic[nr1]-'A'+1);
	}

	for (nr2=0;nr2<ile_faktow;nr2++)
    {
		START:;
		strcpy(x_copy,logic);
		for (nr1=0;nr1<(int) strlen(x_copy);nr1++)
		{
			if (x_copy[nr1]=='A'+nr2) x_copy[nr1]='x';
		}

		for (nr1=0;nr1<9;nr1++)
		{
//continue;
			ptr=strstr(x_copy,table[nr1].opis);
			if (ptr!=NULL)
			{
	      	// jesli nalezy dopisac negacje
				if (table[nr1].result!='0' && table[nr1].result!='!')
				{
//DeniseReport(handle,"1");
					if (ptr==x_copy)
					{
						sprintf(temp,"%c%c",table[nr1].result,logic[ptr-x_copy+table[nr1].poz]);
					}
					else
					{
						sprintf(temp,"%.*s%c%c%s",ptr-x_copy,logic,table[nr1].result,logic[ptr-x_copy+table[nr1].poz],logic+strlen(x_copy)-strlen(ptr)-strlen(table[nr1].opis));
					}
				}
	         // jesli nie dopisujemy negacji
				else if (table[nr1].result=='0')
				{
					// czy w ogole cos dopisujemy?
				    if (table[nr1].poz!=-1)
		            {
						if (ptr==x_copy)
						{
							sprintf(temp,"%c",logic[table[nr1].poz]);
						}
						else
						{
							sprintf(temp,"%.*s%c%s",ptr-x_copy,logic,logic[ptr-x_copy+table[nr1].poz],logic+strlen(x_copy)-strlen(ptr)-strlen(table[nr1].opis));
						}
//DeniseReport(handle,"cos jest: %.*s | %c | %s",ptr-x_copy,logic,logic[ptr-x_copy+table[nr1].poz],logic+strlen(x_copy)-strlen(ptr)-strlen(table[nr1].opis));
					}
					else
					{
						if (ptr==x_copy)
						{
							sprintf(temp,"");
						}
						else
						{
							sprintf(temp,"%.*s%s",ptr-x_copy,logic,logic+strlen(x_copy)-strlen(ptr)-strlen(table[nr1].opis));
						}
					}
				}
				// jesli sprzecznosc!
				else if (table[nr1].result=='!')
				{
		         	sprintf(temp,"");
//DeniseReport(handle,"sprzecznosc!");
					return -1;
			    }
	  			strcpy(logic,temp);

//DeniseReport(handle,"!%s!...",logic);

//DeniseReport(handle,"!%s!",logic+strlen(x_copy)-strlen(ptr)-strlen(table[nr1].opis));
				goto START;
			}
		}
	}
	// PRAWDA
	if (logic[0]=='\0') return 1;
	strcpy(temp,logic);

//DeniseReport(handle,"logikaC: '%s'",logic);
   log_expr2clause(handle,temp,logic);

//printf("zdanie: %s\n",temp1);

   fact->ile_klauzul=glo_count_parts(handle,logic,',');
   free(fact->logic);
   fact->logic=(char **) malloc(fact->ile_klauzul*sizeof(char *));

   for (nr1=0;nr1<fact->ile_klauzul;nr1++)
   {
      glo_get_part(logic,nr1+1,temp,',');
      fact->logic[nr1]=(char *) malloc(strlen(temp)+1);
      memcpy(fact->logic[nr1],temp,strlen(temp)+1);
   }

	return 0;
}

static int count_logic_facts(char *wyrazenie)
{
   int nr1,ile=0;

   for (nr1=0;nr1<(int) strlen(wyrazenie);nr1++)
      if (wyrazenie[nr1]>='A' && wyrazenie[nr1]<='Z') ile++;

   return ile;
}
