// ***************************************************************************************
//                                            MYTH
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static int input_output_analysis(struct DeniseHandle *handle);
static void make_busy(struct DeniseHandle* handle,char *orig,char *busy);
static int get_sentence_type(char *orig);
static BOOL is_identifier(struct DeniseHandle *handle,char *result);
static int conv_get_conjunction(struct DeniseHandle *handle,char *expr);
static int conv_perform_action(int *action_table,int ile_action,int action);
static BOOL meaning_analysis(struct DeniseHandle *handle);

// ***************************************************************************************
// EXTERN                              BOOL myth(void)
// ***************************************************************************************
BOOL myth(struct DeniseHandle* handle)
{
   /*
   <analiza wstepna>
   * opcjonalna identyfikacja
   - jesli jest to tryb identyfikacji:
      - wylaczamy tryb identyfikacji,
      - sprawdzamy, czy wprowadzona wypowiedz stanowi identyfikacje,
      - jesli stanowi, to:
         - dodajemy nowe identyfikatory do pamietanego w "cache" faktu, do pierwszego
           faktu prostego z tematem '-1' do pola id_subject,
           uwzgledniajac optymalizacje identyfikatorow,
         - przenosimy fakt z cache do funkcji, (zerujemy wskaznik w cache)
         - skaczemy do analizy tematycznej,
   * sprawdzamy, czy wprowadzony tekst jest wyrazeniem logicznym,
   - jesli tak, to:
      - sprowadzamy je do klauzul,
      - wypisujemy klauzule,
      - wychodzimy z funkcji,
   - jesli nie, dalej,
   * tworzymy format wypowiedzi bez znakow interpunkcyjnych, z malymi literami itp.
   - jesli w wyniku uproszczenia wypowiedz utracila wszelka informacje, to wychodzimy z funkcji,
	* opcjonalne wyjasnianie
   - jesli jest to tryb wyjasniania:
      - sprawdzamy, czy wprowadzona wypowiedz stanowi wyjasnienie,
      - jesli tak, to:
      	- eliminujemy bledne interpretacje zdania (kasujemy fakty),
		  	- jesli pozostal jeden rozbior:
         	- wylaczamy ten tryb,
            - skaczemy do interpretacji spojnikow,
         - jesli wiecej rozbiorow to:
	      	- ustawiamy tryb wyjasniania o co chodzi w zdaniu,
	         - wywolujemy funkcje gramatyczna, ktora ma pomoc w identyfikacji
	           wypowiedzi,
	         - wychodzimy z funkcji w tym trybie,
		- jesli nie, to:
      	- kasujemy liste pamietanych faktow,
         - wylaczamy tryb wyjasniania,
   * wydobywamy podstawowe informacje:
   - wypowiedz w formie pytania/twierdzenia,
   - format oczekiwanej odpowiedzi (brak, odpowiedz jednozdaniowa, dowod),
   * jesli uproszczony format wypowiedzi po wykonaniu operacji jest pusty,
     to sprawdzamy 1) czy zawarto w wypowiedzi oryginalnej informacje
     o formacie oczekiwanej wypowiedzi 2) i czy w pamieci podrecznej istnieje
     ostatnio wyprowadzony dowod, lub jest to tryb identyfikacji,
   - jesli tak, to:
      - w odpowiedni (oczekiwany) sposob odpowiadamy,
   - jesli nie, to:
      - wychodzimy z funkcji,

   <analiza operacyjna>
	* sprawdzenie, czy wypowiedz jest pytaniem/twierdzeniem znaczen tozsamosciowych
	- jesli TAK, to wychodzimy z funkcji,
	- jesli NIE, to dalej

   <analiza gramatyczna>
   * dokonywana jest analiza gramatyczna zdania:
   - jesli powiedzie sie, i wynikiem jest lista faktow reprezentujacych zdanie, to:
      - jesli byl tryb identyfikacji to wychodzimy z niego:
         - zwalniamy pamiec faktu w "cache", jesli byl,
		- jesli jest tylko jeden mozliwy rozbior, to zdanie jest zidentyfikowane,
      - jesli zas mozliwych rozbiorow jest wiecej:
      	- ustawiamy tryb wyjasniania o co chodzi w zdaniu,
         - wywolujemy funkcje gramatyczna, ktora ma pomoc w identyfikacji
           wypowiedzi,
         - wychodzimy z funkcji w tym trybie,
   - jesli nie powiedzie sie, to:
      - wychodzimy z funkcji,
	* przeprowadzana jest interpretacja pod katem uzytych spojnikw w wyrazeniach:
	- jesli nie powiedzie sie, to:
		- wychodzimy z funkcji
   * ostatecznie probuje sie zoptymalizowac wypowiedz i wyeliminowac zbedne fakty proste
  	- jesli zdanie jest wewnetrznie sprzeczne, to:
   	- informujemy i wychodzimy z funkcji

   <analiza tematyczna>
   * dokonujemy scislej identyfikacji faktu na podstawie jego identyfikatorow:
   - pobieramy kolejne fakty proste z badanego faktu,
      - dokonujemy scislej identyfikacji tego faktu prostego,
         - jesli nie udalo sie zidentyfikowac faktu, to:
            - wpisujemy jako numer faktu 0 (oznacza brak identyfikacji),
            - nastepny fakt prosty,
         - jesli zidentyfikowano dokladnie jeden temat, to:
            - wpisujemy jego numer do faktu prostego,
            - kasujemy zwrocona liste,
            - nastepny fakt prosty,
         - jesli zidentyfikowano wiecej takich tematow, to:
            - wpisujemy do faktu prostego liczbe '-1',
            - informujemy o problemie zidentyfikowania tematu,
            - ustawiamy tryb identyfikacji,
            - zapamietujemy fakt w "cache",
            - kasujemy zwrocona liste,
            - wychodzimy z funkcji,
         - jesli zidentyfikowano wiecej tematow podobnych, to:
            - wpisujemy do faktu prostego liczbe '-1',
            - informujemy o problemie zidentyfikowania tematu,
            - ustawiamy tryb identyfikacji,
            - zapamietujemy fakt w "cache",
            - kasujemy zwrocona liste,
            - wychodzimy z funkcji,

   <analiza konwersacyjna>
   - czy temat fs wskazuje na Denise lub to osoba 2?
      - jesli TAK:
         - temat musi wskazywac na Denise i musi to byc osoba 2 (jak nie to ACTION)
         - jesli to twierdzenie:
            - musi to byc tryb SUPERVISOR (jak nie to ACTION),
         - skok do: analiza logiczna
      - jesli NIE - dalej
   - czy temat fs wskazuje na rozmowce lub to osoba 1?
      - jesli TAK:
         - jesli to twierdzenie:
            - jesli to pierwszy rozmowca, to witamy,
            - jesli zmienil sie rozmowca, to zegnamy starego i witamy,
            - jesli mozna zapamietujemy temat rozmowcy,
	      - skok do: analiza logiczna
		- jesli NIE - dalej
   - czy temat fs wskazuje na temat lub to osoba 3?
      - jesli TAK:
         - jesli to pierwszy lub zmienil sie temat, to zapamietujemy,
			- skok do: analiza logiczna

   <analiza logiczna>
	* odnajdujemy w tabeli konwersacyjnej odpowiednia pozycje,
     odpowiadajaca zaistnialej sytuacji,
   * dokonujemy analizy logicznej:
   - jesli spelnione sa warunki pozycji konwersacyjnej,
     i wymagana jest akcja odpowiadania,
     to informujemy uzytkownika o wynikach analizy logicznej w odpowiedniej formie,
     tj. pamietajac o oczekiwanej formie odpowiedzi,
   - jesli w wyniku analizy stwierdzono brak danych, to kasujemy
     aktualnie posiadane listy dowodowe w pamieci "cache",
   - a jesli w wyniku analizy powstaly klauzule dowodowe, to zapamietujemy je,
   * kasujemy liste tematow z pamieci podrecznej,

   <opcjonalne zapamietanie faktu>
   * opcjonalne zapamietanie faktu:
   - jesli spelnione sa warunki pozycji konwersacyjnej, i nie uzyskano sprzecznosci, oraz
     w badanym fakcie istnieja fakty proste w scisly sposob nie znane w calej bazie, to:
      - zapamietujemy fakt opierajac sie na wczesniejszej identyfikacji scislej,

   <opcjonalne zadanie pytania>
	-
   */

   struct Fact *fact,*fact_next;
   struct FactSimple *fs;
   struct ThemesList *identified_themes=NULL; // lista zidentyfikowanych tematow
   struct ThemesList *themes;
   struct Theme *theme,*AboutTheme,*FromTheme;
	struct ConvData *conv; // pozycja w tablicy konwersacyjnej
   struct LogicAnalysis logic_result; // wynik analizy logicznej
	BOOL unknown_theme; // ustawiana jesli istnieje nieznany temat w wypowiedzi
   char temp1[10000],temp2[10000],temp3[10000],*ptr;
   int nr1,nr2,nr3,nr4,ile,con;
   int action[200],ile_action,action_arg[200][10],ile_arg[200],action_nr; // akcje, i ich argumenty

//test_decl(handle);
//return FALSE;

   handle->syntax.busy[0]='\0';
   logic_result.logic_state=LOGIC_FALSE;
   logic_result.neg=FALSE;
	ile_action=0; // brak akcji

	//       *******************************************************
   //                    A N A L I Z A   W S T E P N A
   //       *******************************************************

/*	sprintf(handle->syntax.busy,"marek jest niemiy");
	fact=grammar_analysis(handle);
	fact=grammar_analysis(handle);
	fact=grammar_analysis(handle);
	fact=grammar_analysis(handle);
	fact=grammar_analysis(handle);
return TRUE;
*/
	if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PRELIMINARY_ANALYSIS,"* analiza wstepna"));

   // jesli jest to tryb identyfikacji:
   if (handle->cache.identify_mode==TRUE)
   {
		if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_IDENTIFICATION_MODE,"wykryto tryb identyfikacji"));

      // wylaczamy tryb identyfikacji
      handle->cache.identify_mode=FALSE;

      // sprawdzamy, czy wprowadzona wypowiedz stanowi identyfikacje,
      // jesli stanowi, to:
      if (is_identifier(handle,temp1)==TRUE)
      {
//printf("ID\n");
			if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_IDENTIFICATION_SENTENCE,"wypowiedz rozpoznana jako identyfikacja"));

         // przenosimy fakt z cache do funkcji, (zerujemy wskaznik w cache)
         handle->cache.facts=handle->cache.temp_fact;
         handle->cache.temp_fact=NULL;

         // dodajemy nowe identyfikatory do pamietanego w "cache" faktu, do pierwszego
         // faktu prostego z tematem '-1' do pola id_subject,
         // uwzgledniajac optymalizacje identyfikatorow,
         for (fs=handle->cache.facts->fs;fs!=NULL;fs=fs->next)
         {
            if (fs->theme_number==-1)
            {
               sprintf(temp2,"%s,%s",fs->sentence->id_subject,temp1);
               ide_optimize_identifiers(handle,temp2);
               free(fs->sentence->id_subject);
               fs->sentence->id_subject=malloc(strlen(temp2)+1);
               memcpy(fs->sentence->id_subject,temp2,strlen(temp2)+1);

               // i dalej do identyfikacji tematu
               goto THEME_IDENTIFICATION;
            }
         }
      }
		else
		{
			if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_NO_IDENTIFICATION_SENTENCE,"wypowiedz nie zostala rozpoznana jako identyfikacja"));
		}
   }

   // * sprawdzamy, czy wprowadzony tekst jest wyrazeniem logicznym,
   // jesli tak, to:
   if (log_islogic(handle,handle->syntax.orig)==TRUE)
   {
      // sprowadzamy je do klauzul,
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_LOGIC_CLAUSE,"rozpoznano wyrazenie logiczne"));
      log_descr2expr(handle,handle->syntax.orig,temp1);
//DeniseReport(handle,"1: '%s'\n",temp1);
		if (log_expr_simplification(temp1,temp2)==0)
		{
//DeniseReport(handle,"2: '%s'",temp2);
			log_expr2clause(handle,temp2,temp1);
//DeniseReport(handle,"3: '%s'",temp1);
	        // wypisujemy klauzule,
			log_expr_description(handle,temp1,temp2);
//DeniseReport(handle,"4: '%s'",temp2);
			DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_LOGIC_CLAUSE_FORM,"po sprowadzeniu do klauzul"));
			DeniseMessage(handle,"%s",temp2);
		}
		else
		{
			DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_LOGIC_CLAUSE_FORM,"po sprowadzeniu do klauzul"));
			DeniseMessage(handle,"blad: akceptuje max. 1 XOR i 1 THEN");
		}
	    // wychodzimy z funkcji,
		return TRUE;
   }

   // * tworzymy format wypowiedzi bez znakow interpunkcyjnych, z malymi literami itp.
   make_busy(handle,handle->syntax.orig,handle->syntax.busy);

   // jesli w wyniku uproszczenia wypowiedz utracila wszelka informacje, to wychodzimy z funkcji,
   if (strlen(handle->syntax.busy)==0)
   {
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_EMPTY_SENTENCE,"wypowiedz nie zawiera zadnej informacji"));
      return TRUE;
   }

	// * opcjonalne wyjasnianie
   // - jesli jest to tryb wyjasniania:
   if (handle->cache.explain_mode==TRUE)
	{
   	// sprawdzamy, czy wprowadzona wypowiedz stanowi wyjasnienie,
		// jesli tak, to:
      if (gra_is_explanation(handle)==TRUE)
      {
			// jesli pozostal jeden rozbior:
			if (handle->cache.facts->next==NULL)
			{
		   	// wylaczamy ten tryb,
            handle->cache.explain_mode=FALSE;
			   // skaczemy do interpretacji spojnikow,
            goto CONJUNCTIONS_ANALYSIS;
         }
		   // jesli wiecej rozbiorow to:
			else
         {
				// ustawiamy tryb wyjasniania o co chodzi w zdaniu,
            handle->cache.explain_mode=TRUE;
      		// wywolujemy funkcje gramatyczna, ktora ma pomoc w identyfikacji
				// wypowiedzi,
				gra_help_explain(handle);
				// wychodzimy z funkcji w tym trybie,
            return TRUE;
         }
   	}
		//	jesli nie, to:
      else
      {
	   	// kasujemy liste pamietanych faktow,
			for (fact=handle->cache.facts;fact!=NULL;fact=fact_next)
         {
         	fact_next=fact->next;
            dat_fact_free(fact);
            free(fact);
         }
			handle->cache.facts=NULL;
	   	// wylaczamy tryb wyjasniania,
		   handle->cache.explain_mode=FALSE;
   	}
   }

   // * wydobywamy podstawowe informacje:
   if (input_output_analysis(handle)==0) handle->syntax.busy[0]='\0';

   // * jesli uproszczony format wypowiedzi po wykonaniu operacji jest pusty, to sprawdzamy:
   if (strlen(handle->syntax.busy)==0)
   {
      // 1) czy zawarto w wypowiedzi oryginalnej informacje o formacie oczekiwanej wypowiedzi
      if (handle->syntax.prefered_output!=NO_PREFERED)
      {
         // 2) i czy w pamieci podrecznej istnieje ostatnio wyprowadzony dowod,
         // jesli tak, to:
         if (handle->cache.last_list!=NULL)
         {
            // w odpowiedni (oczekiwany) sposob odpowiadamy,
            switch (handle->syntax.prefered_output)
            {
               case PREFERED_PROVE:
					if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_PROVE,"oczekiwana odpowiedz w formie dowodu"));
                  log_prove_explanation(handle,handle->cache.last_list);
                  break;
               case PREFERED_RESULT:
						if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_NO_PROVE,"oczekiwana odpowiedz bez dowodu"));
                  log_result_explanation(handle,handle->cache.last_list);
                  break;
            }

            // wychodzimy z funkcji
            return TRUE;
         }
         // lub jest to tryb identyfikacji,
         else if (handle->cache.identify_mode==TRUE && handle->cache.temp_fact!=NULL)
         {
            for (fs=handle->cache.temp_fact->fs;fs!=NULL;fs=fs->next)
               if (fs->theme_number==-1) break;

            // dokonujemy scislej identyfikacji dla faktu prostego z tematem -1
            identified_themes=NULL;
            ide_find_theme_of_fs(handle,&identified_themes,fs,MODE_EXACT);

            // zbieramy wszystkie identyfikatory znalezionych tematow,
            temp1[0]='\0';
            for (themes=identified_themes;themes!=NULL;themes=themes->next)
            {
               theme=themes->theme;
               if (theme==NULL) continue;
               // pobieramy tylko te, ktore nie sa zanegowane
               for (nr1=0,nr2=glo_count_parts(handle,theme->identify,',');nr1<nr2;nr1++)
               {
                  glo_get_part(theme->identify,nr1+1,temp2,',');
                  if (temp2[0]=='!') continue;

                  if (temp1[0]!='\0')
                  {
                     sprintf(temp3,",");
                     strcat(temp1,temp3);
                  }
                  strcat(temp1,temp2);
               }
               ide_optimize_identifiers(handle,temp1);
            }

            // kasujemy z listy identyfikatory faktu prostego
            for (nr1=0,nr2=glo_count_parts(handle,fs->sentence->id_subject,',');nr1<nr2;nr1++)
            {
               ide_filter_identifiers(handle,fs->sentence->id_subject,temp3,0);
               glo_get_part(temp3,nr1+1,temp2,',');
               if (temp2[0]=='!') continue;

               ide_filter_identifiers(handle,temp1,temp3,0);
               nr3=glo_get_part_position(1,temp3,temp2,',');
               if (nr3<=0) continue;

               glo_delete_part_sep(handle,temp1,nr3,',');
            }

            ide_filter_identifiers(handle,temp1,temp2,1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_EXPLAIN_PLEASE,"wyjasnij, czy to %s"),temp2);
            return TRUE;
         }
         // jesli nie, to:
         else
         {
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_QUESTION_ABOUT,"o co pytasz?"));
            // wychodzimy z funkcji,
            return TRUE;
         }
      }
      else
      {
			DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_SENTENCE_WITH_NO_SENSE,"nie widze sensu w tej wypowiedzi"));
         // wychodzimy z funkcji,
         return TRUE;
      }
   }

	//       *******************************************************
	//                 A N A L I Z A   O P E R A C Y J N A
	//       *******************************************************

	// * sprawdzenie, czy wypowiedz jest pytaniem/twierdzeniem znaczen tozsamosciowych
	// jesli TAK, to wychodzimy z funkcji
	if (meaning_analysis(handle)==TRUE) return TRUE;
	// jesli NIE, to dalej

	//       *******************************************************
	//                A N A L I Z A   G R A M A T Y C Z N A
	//       *******************************************************

   // * dokonywana jest analiza gramatyczna zdania:
	if (handle->viewparams.words==ON || handle->viewparams.clause==ON || handle->viewparams.syntax==ON || handle->viewparams.syntax_time==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_GRAMMAR_ANALYSIS,"* analiza gramatyczna"));

	// kasujemy stara liste faktow jesli byla
	if (handle->cache.facts!=NULL)
   {
    	dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
        handle->cache.facts=NULL;
	}

	// jesli powiedzie sie, i wynikiem jest fakt reprezentujacy zdanie, to:
	handle->cache.facts=grammar_analysis(handle);
   if (handle->cache.facts!=NULL)
   {
	   // jesli byl tryb identyfikacji to wychodzimy z niego,
      handle->cache.identify_mode=FALSE;
      // zwalniamy pamiec faktu w "cache", jesli byl,
      if (handle->cache.temp_fact!=NULL)
      {
         dat_fact_free(handle->cache.temp_fact);
			free(handle->cache.temp_fact);
         handle->cache.temp_fact=NULL;
      }

		// jesli jest tylko jeden mozliwy rozbior, to zdanie jest zidentyfikowane,
      // jesli zas mozliwych rozbiorow jest wiecej:
      if (handle->cache.facts->next!=NULL)
      {
	      //	ustawiamy tryb wyjasniania o co chodzi w zdaniu,
	      handle->cache.explain_mode=TRUE;
	      // wywolujemy funkcje gramatyczna, ktora ma pomoc w identyfikacji wypowiedzi,
			gra_help_explain(handle);
	      // wychodzimy z funkcji w tym trybie,
	      return TRUE;
      }
   }
   // jesli nie powiedzie sie, to:
   else
   {
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_GRAMMAR_ANALYSIS_FAILED,"analiza gramatyczna nie powiodla sie"));
      return TRUE;
   }

   // skoro analiza powiodla sie, to kasujemy liste klauzul z ostatniego wnioskowania
   if (handle->cache.last_list!=NULL)
   {
      log_clauses_free(&(handle->cache.last_list));
//		free(handle->cache.last_list);
      handle->cache.last_list=NULL;
   }

//return TRUE;

//DeniseMessage(handle,"1 ILE KLAUZUL: %d",handle->cache.facts->ile_klauzul);

	// * przeprowadzana jest interpretacja pod katem uzytych spojnikow w wyrazeniach:
	CONJUNCTIONS_ANALYSIS:;
	if (handle->viewparams.conjunctions_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_CONJUNCTIONS_ANALYSIS,"* interpretacja spojnikow"));

	// analiza spojnikow
	if (gra_conjunctions_analysis(handle,handle->cache.facts)==FALSE)
	{
		// jesli nie powiedzie sie, to:
		// wychodzimy z funkcji
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_CONJUNCTIONS_ANALYSIS_FAILED,"analiza spojnikow nie powiodla sie"));
      dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
		handle->cache.facts=NULL;
		return TRUE;
	}

//DeniseMessage(handle,"2 ILE KLAUZUL: %d",handle->cache.facts->ile_klauzul);

   // * ostatecznie probuje sie zoptymalizowac wypowiedz i wyeliminowac zbedne fakty proste

	nr1=log_fact_simplification(handle,handle->cache.facts);
   	// jesli zdanie jest wewnetrznie sprzeczne, to informujemy i wychodzimy
	if (nr1==-1)
   {
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_CONTRADICTORY_SENTENCE,"wprowadzone zdanie jest wewnetrznie sprzeczne"));
   	dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
      handle->cache.facts=NULL;
		return TRUE;
   }
   	// jesli zdanie jest zawsze prawda (not A or A) to informujemy i wychodzimy
	else if (nr1==1)
   {
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ALWAYS_TRUE_SENTENCE,"wprowadzone zdanie jest tautologia"));
   	dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
      handle->cache.facts=NULL;
		return TRUE;
   }

//DeniseMessage(handle,"3 ILE KLAUZUL: %d",handle->cache.facts->ile_klauzul);

   //       *******************************************************
   //                 A N A L I Z A   T E M A T Y C Z N A
   //       *******************************************************
   THEME_IDENTIFICATION:
//return TRUE;

	if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_THEME_ANALYSIS,"* analiza tematyczna"));

   // * dokonujemy scislej identyfikacji faktu na podstawie jego identyfikatorow:
   // pobieramy kolejne fakty proste z badanego faktu,
	unknown_theme=FALSE;
   for (fs=handle->cache.facts->fs;fs!=NULL;fs=fs->next)
   {
      if (fs->sentence->id_subject==NULL || strlen(fs->sentence->id_subject)==0)
      {
			DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_SUBJECT_NOT_IDENTIFIED,"podmiot w zaden sposob nie moze zostac zidentyfikowany"));
         handle->cache.identify_mode=FALSE;
         return TRUE;
      }
//printf("zdanie: %s, osoba: %d\n",fs->sentence->orig,ext_get_person(handle,fs->sentence));

      // dokonujemy scislej identyfikacji tego faktu prostego,
      switch (ide_find_theme_of_fs(handle,&identified_themes,fs,MODE_EXACT))
      {
         // jesli nie udalo sie zidentyfikowac faktu, to:
         case IDENTIFIED_NOTHING:
            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
				if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_THEME_NOT_FOUND,"temat '%s' nie istnieje w bazie"),temp1);
            // wpisujemy jako numer faktu 0 (oznacza brak identyfikacji),
            fs->theme_number=0;
            fs->databaseID[0]='\0';
				unknown_theme=TRUE;
            // nastepny fakt prosty,
            break;
         // jesli zidentyfikowano dokladnie jeden temat, to:
         case IDENTIFIED_ONE:
            ide_filter_identifiers(handle,identified_themes->theme->identify,temp1,1);
            if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_THEME_IDENTIFIED,"temat '%s' zostal scisle zidentyfikowany"),temp1);
            // wpisujemy jego numer do faktu prostego,
            strcpy(fs->databaseID,identified_themes->theme->database->databaseID);
            fs->theme_number=identified_themes->theme->number;
            // kasujemy zwrocona liste,
            ide_themes_list_free(identified_themes);
            identified_themes=NULL;
            // nastepny fakt prosty,
            break;
         // jesli zidentyfikowano wiecej takich tematow, to:
         case IDENTIFIED_MORE_SAME:
            // informujemy o problemie zidentyfikowania tematu i ew. pomagamy,
            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NEED_IDENTIFICATION,"temat '%s' musi zostac lepiej zdefiniowany"),temp1);

            // wpisujemy do faktu prostego liczbe '-1',
            fs->theme_number=-1;
            fs->databaseID[0]='\0';

            // ustawiamy tryb identyfikacji,
            handle->cache.identify_mode=TRUE;

            // zapamietujemy fakt w "cache",
            handle->cache.temp_fact=handle->cache.facts;
            handle->cache.facts=NULL;

            // kasujemy zwrocona liste,
            ide_themes_list_free(identified_themes);
            identified_themes=NULL;

            // wychodzimy z funkcji,
            return TRUE;
         // jesli zidentyfikowano wiecej tematow podobnych, to:
         case IDENTIFIED_MORE_ALIKE:
            // informujemy o problemie zidentyfikowania tematu i ew. pomagamy,
            // zbieramy wszystkie identyfikatory znalezionych tematow,
            temp1[0]='\0';
            for (themes=identified_themes;themes!=NULL;themes=themes->next)
            {
               theme=themes->theme;
               if (theme==NULL) continue;
               // pobieramy tylko te, ktore nie sa zanegowane
               for (nr1=0,nr2=glo_count_parts(handle,theme->identify,',');nr1<nr2;nr1++)
               {
                  glo_get_part(theme->identify,nr1+1,temp2,',');
                  if (temp2[0]=='!') continue;

                  if (temp1[0]!='\0')
                  {
                     sprintf(temp3,",");
                     strcat(temp1,temp3);
                  }
                  strcat(temp1,temp2);
               }
               ide_optimize_identifiers(handle,temp1);
            }

            // kasujemy z listy identyfikatory faktu prostego
            for (nr1=0,nr2=glo_count_parts(handle,fs->sentence->id_subject,',');nr1<nr2;nr1++)
            {
               ide_filter_identifiers(handle,fs->sentence->id_subject,temp3,0);
               glo_get_part(temp3,nr1+1,temp2,',');
               if (temp2[0]=='!') continue;

               ide_filter_identifiers(handle,temp1,temp3,0);
               nr3=glo_get_part_position(1,temp3,temp2,',');
               if (nr3<=0) continue;

               glo_delete_part_sep(handle,temp1,nr3,',');
            }
            ide_filter_identifiers(handle,temp1,temp2,2);
            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,2);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_WHAT_THEME_QUESTION,"co za %s? %s, czy jak?"),temp1,temp2);

            // wpisujemy do faktu prostego liczbe '-1',
            fs->theme_number=-1;
            fs->databaseID[0]='\0';

            // ustawiamy tryb identyfikacji,
            handle->cache.identify_mode=TRUE;

            // zapamietujemy fakt w "cache",
            handle->cache.temp_fact=handle->cache.facts;
            handle->cache.facts=NULL;

            // kasujemy zwrocona liste,
            ide_themes_list_free(identified_themes);
            identified_themes=NULL;

            // wychodzimy z funkcji,
            return TRUE;
      }
   }

   //       *******************************************************
   //              A N A L I Z A   K O N W E R S A C Y J N A
   //       *******************************************************
   // * analiza konwersacyjna

	if (handle->viewparams.interlocutory_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INTERLOCUTORY_ANALYSIS,"* analiza konwersacyjna"));

//DeniseMessage(handle,"AAAAAA");

   // pobieramy kolejne fakty proste
   for (fs=handle->cache.facts->fs;fs!=NULL;fs=fs->next)
   {
      nr1=ext_get_person(handle,fs->sentence);
      if (fs->theme_number>0) theme=ide_find_theme(handle,fs->databaseID,fs->theme_number);
      else theme=NULL;

      // czy temat fs wskazuje na Denise lub to osoba 2?
      // jesli TAK:
      if (nr1==2 || (fs->theme_number>0 && theme==handle->cache.denise))
      {
         // temat musi wskazywac na Denise i musi to byc osoba 2 (jak nie to ACTION)
         if (nr1==1)
         {
            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_IT_IS_ME_DENISE,"to ja jestem %s"),temp1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_USE_OTHER_DESCRIPTION,"musisz inaczej siebie opisac"),temp1);
            goto ACTION;
         }
         else if (nr1==3)
         {
            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_IT_IS_ME_DENISE,"to ja jestem %s"),temp1);
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_TALK_IN_SECOND_PERSON,"mow do mnie w drugiej osobie"));
            goto ACTION;
         }
         if ((fs->theme_number==0 && handle->cache.denise!=NULL) || (fs->theme_number>0 && theme!=handle->cache.denise && handle->cache.denise!=NULL))
         {
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_WRONG_DENISE_IDENTIFICATION,"okreslenie Denise nie jest zgodne z posiadanym w bazie"));
            goto ACTION;
         }

         // jesli to twierdzenie:
         if (handle->syntax.input_type==ZDANIE_OZNAJMUJACE)
         {
            // musi to byc tryb SUPERVISOR (jak nie to ACTION),
            if (handle->Supervisor==OFF)
            {
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NOT_SUPERVISOR_MODE,"nie masz uprawnien do charakteryzowania Denise"));
               goto ACTION;
            }
         }
      }
      // czy temat fs wskazuje na rozmowce lub to osoba 1?
      // jesli TAK:
      else if (nr1==1 || (fs->theme_number>0 && theme==handle->cache.user))
      {
         // jesli to twierdzenie:
         if (handle->syntax.input_type==ZDANIE_OZNAJMUJACE)
         {
            // jesli nie bylo tematu rozmowcy, to dokonujemy powitania,
            if (handle->cache.user==NULL)
            {
               ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_HELLO_NEW_USER,"witaj %s"),temp1);
            }
            // jesli temat rozmowcy zostaje zmieniony, to dokonujemy pozegnania i powitania,
            else if (fs->theme_number==0 || handle->cache.user!=theme)
            {
               ide_filter_identifiers(handle,handle->cache.user->identify,temp1,1);
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_BYE_USER,"zegnaj %s"),temp1);

               ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_HELLO_NEW_USER,"witaj %s"),temp1);
            }

            // jesli mozna zapamietujemy temat Denise,
            if (fs->theme_number>0) handle->cache.user=theme;
         }
      }
		// czy temat fs wskazuje na temat lub to osoba 3?
      // jesli TAK:
	   else if (nr1==3 || (fs->theme_number>0 && theme==handle->cache.theme))
		{
         // jesli mozna zapamietujemy aktualny temat,
         if (fs->theme_number>0) handle->cache.theme=theme;
		}
   }

   //       *******************************************************
   //                    A N A L I Z A   L O G I C Z N A
   //       *******************************************************
//   LOGIC_ANALYSIS:;

	if (handle->viewparams.logic_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOGIC_ANALYSIS,"* analiza logiczna"));

	// * odnajdujemy w tabeli konwersacyjnej odpowiednia pozycje, odpowiadajaca zaistnialej sytuacji,
   for (conv=handle->logic.conversation;conv!=NULL;conv=conv->next)
   {
   	// sprawdzenie warunkow
      // identyfikujemy spojnik wystepujacy w wyrazeniu
		con=conv_get_conjunction(handle,conv->conditions);

		// pobieramy kolejne wyrazy
		ile=glo_count_parts(handle,conv->conditions,' ');
      for (nr1=0;nr1<ile;nr1++)
      {
      	glo_get_part(conv->conditions,nr1+1,temp1,' ');
         switch (atoi(temp1))
         {
				// pomijamy spojniki
				case CONV_AND:
				case CONV_OR: break;
            // wlasciwe warunki
				case CONV_USER_ANNOUNCE:
					// jesli spelnione
					if (handle->syntax.input_type==ZDANIE_OZNAJMUJACE)
               {
               	// jesli spojnik OR to mozna zakonczyc
                  if (con==CONV_OR) goto FOUNDED;
               }
               // jesli nie spelnione
            	else
               {
						// jesli spojnik AND to warunki nie sa spelnione
                  if (con==CONV_AND) goto NEXT_POSITION;
               }
            	break;
         	case CONV_USER_QUESTION:
					// jesli spelnione
					if (handle->syntax.input_type==ZDANIE_PYTAJACE)
               {
               	// jesli spojnik OR to mozna zakonczyc
                  if (con==CONV_OR) goto FOUNDED;
               }
               // jesli nie spelnione
            	else
               {
						// jesli spojnik AND to warunki nie sa spelnione
                  if (con==CONV_AND) goto NEXT_POSITION;
               }
            	break;
         	case CONV_DENISE_EXIST:
					// jesli spelnione
					if (handle->cache.denise!=NULL)
               {
               	// jesli spojnik OR to mozna zakonczyc
                  if (con==CONV_OR) goto FOUNDED;
               }
               // jesli nie spelnione
            	else
               {
						// jesli spojnik AND to warunki nie sa spelnione
                  if (con==CONV_AND) goto NEXT_POSITION;
               }
            	break;
         	case CONV_USER_EXIST:
					// jesli spelnione
					if (handle->cache.user!=NULL)
               {
               	// jesli spojnik OR to mozna zakonczyc
                  if (con==CONV_OR) goto FOUNDED;
               }
               // jesli nie spelnione
            	else
               {
						// jesli spojnik AND to warunki nie sa spelnione
                  if (con==CONV_AND) goto NEXT_POSITION;
               }
            	break;
         	case CONV_THEME_EXIST:
					// jesli spelnione
					if (handle->cache.theme!=NULL)
               {
               	// jesli spojnik OR to mozna zakonczyc
                  if (con==CONV_OR) goto FOUNDED;
               }
               // jesli nie spelnione
            	else
               {
						// jesli spojnik AND to warunki nie sa spelnione
                  if (con==CONV_AND) goto NEXT_POSITION;
               }
            	break;
				default:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_IDENTIFIER,"wykryto bledny identyfikator w opisie warunkow w pliku %s"), LOGIC_CONVERSATION_FILE);
					goto NEXT_POSITION;
            	break;
         }
      }
      // jesli byl spojnik OR, to niestety, nie spelniono zadnego warunku
      if (con==CONV_OR) goto NEXT_POSITION;

      // jesli wszystko spelnione, to wyjscie z petli - znaleziono pozycje
		FOUNDED:
      break;

		NEXT_POSITION:;
   }

   if (conv==NULL) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_UNKNOWN_SITUATION,"nie moge zareagowac: w %s nie opisano zaistnialej sytuacji"), LOGIC_CONVERSATION_FILE);
   else
   {
   	if (handle->viewparams.logic_analysis==ON) DeniseReport(handle,"conditions: %s, action: %s",conv->conditions,conv->action);

		// przetlumaczenie akcji na zespol konkretnych zadan do wykonania
		/*
		- pobieramy kolejne akcje
      - jesli jest parametr to analizujemy go
      - dopisujemy akcje do listy
      */

		// pobieramy kolejne wyrazy
		ile=glo_count_parts(handle,conv->action,' ');
		ile_action=0;
      for (nr1=0;nr1<ile;nr1++)
      {
      	glo_get_part(conv->action,nr1+1,temp1,' ');
         switch (atoi(temp1))
         {
				// pomijamy spojniki
				case CONV_AND: break;
				case CONV_OR:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_OR_CONJUNCTION,"blad! spojnik OR w czesci wykonawczej w pliku %s"), LOGIC_CONVERSATION_FILE);
            	break;
            // wlasciwe warunki
				default:
					// jesli to wyrazenie alternatywne...
               if (temp1[0]=='[')
               {
						// losujemy pozycje (1-100%)
                  nr2=(rand() % 100)+1;

                  // pobieramy kolejne pozycje zliczajac procenty
						memmove(temp1,temp1+1,strlen(temp1));
                  temp1[strlen(temp1)-1]='\0';
						nr4=glo_count_parts(handle,temp1,';');
                  for (nr3=0;nr3<nr4;nr3++)
                  {
				      	glo_get_part(temp1,nr3+1,temp2,';');
                     nr2=nr2-atoi(temp2);
                     // jesli trafilismy na nasz procent, to wychodzimy i wpisujemy te akcje
                     if (nr2<=0) break;
                  }
                  // jesli znaleziono akcje, to dopisujemy
                  // a jesli nie ma takiej akcji, to trudno
						if (nr2<=0)
                  {
					      glo_get_part(temp2,2,temp3,'=');
	   	         	action[ile_action]=atoi(temp3);
                  }
						ptr=strchr(temp3,'(');
               }
               // w przeciwnym razie...
               else
					{
   	         	action[ile_action]=atoi(temp1);
						ptr=strchr(temp1,'(');
					}

					// jesli akcja ma parametry, to badamy je
					// pobieramy argumenty
               ile_arg[ile_action]=0;
               if (ptr!=NULL)
               {
                	ptr++;
                  nr3=glo_count_parts(handle,ptr,',');
                  for (nr2=0;nr2<nr3;nr2++)
                  {
                   	glo_get_part(ptr,nr2+1,temp2,','); // ptr dotyczy temp1 lub temp3
//DeniseReport(handle,"arg%d=%s",nr2+1,temp2);
 							action_arg[ile_action][ile_arg[ile_action]]=atoi(temp2);
                     ile_arg[ile_action]++;
                  }
					}
               ile_action++;
               break;
         }
      }
   }

//for (fs=fact->fs;fs!=NULL;fs=fs->next)
//{
//	printf("FSy: %s\n",fs->sentence->id_subject);
//}
//printf("...\n");

   // * dokonujemy analizy logicznej
   logic_result=logic_analysis(handle,handle->cache.facts);
//return TRUE;
   // ACTION: CONV_DENISE_ANSWER 20
  	if ((action_nr=conv_perform_action(action,ile_action,CONV_DENISE_ANSWER))>-1)
	{
		if (ile_arg[action_nr]==0) nr1=handle->syntax.prefered_output;
      else
      {
      	switch (action_arg[action_nr][0])
         {
				case CONV_ANSWER_PREFERED:
					nr1=handle->syntax.prefered_output;
            	break;
				case CONV_ANSWER_PROVE:
					nr1=PREFERED_PROVE;
            	break;
				case CONV_ANSWER_SIMPLE:
					nr1=PREFERED_RESULT;
            	break;
				default:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_ANSWER_PARAMETER,"blad: nieznany parametr w akcji odpowiedzi"));
               break;
         }
      }

	  // informujemy uzytkownika o jej wynikach w odpowiedniej formie,
	   // tj. pamietajac o oczekiwanej formie odpowiedzi,
	   switch (nr1)
	   {
	      case NO_PREFERED:
	         if (logic_result.logic_state==LOGIC_TRUE) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_TRUE,"to prawda"));
	         else
	         {
	            if (logic_result.neg==FALSE) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_UNKNOWN,"brak danych"));
	            else DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_FALSE,"to falsz (informacja sprzeczna)"));
	         }
	         break;
	      case PREFERED_PROVE:
	         if (logic_result.logic_state==LOGIC_TRUE)
	         {
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_TRUE_BECAUSE,"to prawda, poniewaz"));
	            log_prove_explanation(handle,logic_result.deduce);
	         }
	         else
	         {
	            if (logic_result.neg==FALSE) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_UNKNOWN,"brak danych"));
	            else
	            {
						DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_FALSE_BECAUSE,"to falsz, poniewaz"));
	               log_prove_explanation(handle,logic_result.deduce);
	            }
	         }
	         break;
	      case PREFERED_RESULT:
	         if (logic_result.logic_state==LOGIC_TRUE)
	         {
//					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_TRUE_BECAUSE,"to prawda, poniewaz"));
	            log_result_explanation(handle,logic_result.deduce);
	         }
	         else
	         {
	            if (logic_result.neg==FALSE) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_UNKNOWN,"brak danych"));
	            else
	            {
//						DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_FALSE_BECAUSE,"to falsz, poniewaz"));
	               log_result_explanation(handle,logic_result.deduce);
	            }
	         }
	         break;
	   }
	}

   // jesli w wyniku analizy stwierdzono brak danych, to kasujemy
   // aktualnie posiadane listy dowodowe w pamieci "cache",
   if (logic_result.logic_state==LOGIC_FALSE && logic_result.neg==FALSE && handle->cache.last_list!=NULL)
   {
      log_clauses_free(&(handle->cache.last_list));
      handle->cache.last_list=NULL;
   }

   // jesli w wyniku analizy powstaly klauzule dowodowe, to zapamietujemy je,
   if (logic_result.logic_state==LOGIC_TRUE || (logic_result.logic_state==LOGIC_FALSE && logic_result.neg==TRUE))
   {
      // zabezpieczenie
      if (handle->cache.last_list!=NULL)
      {
         log_clauses_free(&(handle->cache.last_list));
         handle->cache.last_list=NULL;
      }
      // zapamietujemy nowa liste
      handle->cache.last_list=logic_result.deduce;
   }

   //       *******************************************************
   //           O P C J O N A L N E   Z A P A M I E T A N I E
   //       *******************************************************
//   REMEMBER_FACT:;

   // * opcjonalne zapamietanie faktu:
  	if ((action_nr=conv_perform_action(action,ile_action,CONV_DENISE_REMEMBER))>-1)
   {
      // i nie uzyskano sprzecznosci, oraz
      if ((logic_result.logic_state==LOGIC_FALSE && logic_result.neg==FALSE) ||
          (logic_result.logic_state==LOGIC_TRUE && logic_result.neg==FALSE && fac_exist_unknown_fs(handle,handle->cache.facts)==TRUE))
      {
         // w badanym fakcie istnieja fakty proste w scisly sposob nie znane
         // w calej bazie, to zapamietujemy fakt opierajac sie na wczesniejszej
         // identyfikacji scislej,
         // zapamietujemy fakt opierajac sie na wczesniejszej identyfikacji scislej,
//			if (handle->viewparams.fact_analysis==ON) 
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_REMEMBER_NEW_FACT,"zapamietuje brakujace informacje"));
         fac_remember_fact(handle,handle->cache.facts);
			if (unknown_theme==TRUE)
			{
				identified_themes=NULL;
				unknown_theme=FALSE;
			   for (fs=handle->cache.facts->fs;fs!=NULL;fs=fs->next)
			   {
			      // dokonujemy scislej identyfikacji tego faktu prostego,
			      switch (ide_find_theme_of_fs(handle,&identified_themes,fs,MODE_EXACT))
			      {
			         // jesli nie udalo sie zidentyfikowac faktu, to:
			         case IDENTIFIED_NOTHING:
			            ide_filter_identifiers(handle,fs->sentence->id_subject,temp1,1);
							DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_SUBJECT_STILL_UNKNOWN,"niestety, tematu '%s' nadal nie mozna zidentyfikowac"),temp1);

			            // wpisujemy jako numer faktu 0 (oznacza brak identyfikacji),
			            fs->theme_number=0;
			            fs->databaseID[0]='\0';
							unknown_theme=TRUE;
			            // nastepny fakt prosty,
			            break;
			         // jesli zidentyfikowano dokladnie jeden temat, to:
   	   		   case IDENTIFIED_ONE:
			            ide_filter_identifiers(handle,identified_themes->theme->identify,temp1,1);
		   	         if (handle->viewparams.theme_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SUBJECT_NOW_IDENTIFIED,"zapamietano i zidentyfikowano temat '%s'"),temp1);

		      	      // wpisujemy jego numer do faktu prostego,
		         	   strcpy(fs->databaseID,identified_themes->theme->database->databaseID);
		            	fs->theme_number=identified_themes->theme->number;
			            // kasujemy zwrocona liste,
			            ide_themes_list_free(identified_themes);
			            identified_themes=NULL;

							// mikro-analiza konwersacyjna
					      nr1=ext_get_person(handle,fs->sentence);
//DeniseMessage(handle,"BBBBBB LOOP");
					      if (fs->theme_number>0) theme=ide_find_theme(handle,fs->databaseID,fs->theme_number);
					      else theme=NULL;
//DeniseMessage(handle,"BBBBBB OK");
							// czy temat fs wskazuje na temat lub to osoba 3?
				   	   // jesli TAK:
						   if (nr1==3 || (fs->theme_number>0 && theme==handle->cache.theme))
							{
					         // jesli mozna zapamietujemy aktualny temat,
					         if (fs->theme_number>0) handle->cache.theme=theme;
							}
		   	         // nastepny fakt prosty,
		      	      break;
		      	   // jesli zidentyfikowano wiecej takich tematow, to:
		   	      default:
							DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_IDENTIFICATION_PROBLEM,"problemy ze zidentyfikowaniem tematu '%s'"),temp1);
                     break;
               }
			   }
			}
  		   // fakt jest w pamieci, trzeba wyzerowac liste
			handle->cache.facts=NULL;
      }
   }

   //       *******************************************************
   //          O P C J O N A L N E  Z A D A N I E  P Y T A N I A
   //       *******************************************************
   ACTION:

  	if ((action_nr=conv_perform_action(action,ile_action,CONV_DENISE_QUESTION))>-1)
   {
		AboutTheme=NULL;
		FromTheme=NULL;

   	// wybor tematu AboutTheme
      if (ile_arg[action_nr]==1 || ile_arg[action_nr]==3)
      {
      	switch (action_arg[action_nr][0])
         {
         	case CONV_SUBJECT_DENISE:
					AboutTheme=handle->cache.denise;
               break;
				case CONV_SUBJECT_USER:
					AboutTheme=handle->cache.user;
               break;
            case CONV_SUBJECT_THEME:
					AboutTheme=handle->cache.theme;
               break;
				case CONV_SUBJECT_RANDOM:
					AboutTheme=con_select_theme(handle,NULL,TYPE_RANDOM,MODE_NO_MODE);
               break;
				case CONV_SUBJECT_UNKNOWN:
					AboutTheme=con_select_theme(handle,NULL,TYPE_UNKNOWN,MODE_NO_MODE);
               break;
         	case CONV_SUBJECT_KNOWN:
					AboutTheme=con_select_theme(handle,NULL,TYPE_KNOWN,MODE_NO_MODE);
               break;
        		default:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_QUESTION_PARAMETER,"blad: nieznany parametr w akcji zadawania pytania (%s)"),"argument 1");
               AboutTheme=NULL;
               break;
         }
      }

   	// wybor tematu FromTheme
      if (ile_arg[action_nr]==2 || ile_arg[action_nr]==3)
      {
      	nr1=action_arg[action_nr][ile_arg[action_nr]==2 ? 0 : 1];
         nr2=action_arg[action_nr][ile_arg[action_nr]==2 ? 1 : 2];
         switch (nr2)
         {
         	case CONV_WITHOUT_NEG:
					nr2=MODE_WITHOUT_NEG;
            	break;
            case CONV_WITH_NEG:
					nr2=MODE_WITH_NEG;
            	break;
            case CONV_MODE_FREE:
					nr2=MODE_FREE;
            	break;
				default:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_QUESTION_PARAMETER,"blad: nieznany parametr w akcji zadawania pytania (%s)"),"?");
               goto BUILDING_QUESTION;
					break;
         }
      	switch (nr1)
         {
				case CONV_MOST_ALIKE:
					if (AboutTheme!=NULL) FromTheme=con_select_theme(handle,AboutTheme,TYPE_ALIKE,nr2);
               break;
				case CONV_MOST_DIFFERENT:
					if (AboutTheme!=NULL) FromTheme=con_select_theme(handle,AboutTheme,TYPE_DIFFERENT,nr2);
               break;
				case CONV_MOST_KNOWN:
					FromTheme=con_select_theme(handle,AboutTheme,TYPE_KNOWN,nr2);
               break;
				case CONV_MOST_UNKNOWN:
					FromTheme=con_select_theme(handle,AboutTheme,TYPE_UNKNOWN,nr2);
               break;
				case CONV_RANDOM:
					FromTheme=con_select_theme(handle,AboutTheme,TYPE_RANDOM,nr2);
               break;
        		default:
					DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WRONG_QUESTION_PARAMETER,"blad: nieznany parametr w akcji zadawania pytania (%s)"),ile_arg[action_nr]==2 ? "1" : "2");
               FromTheme=NULL;
               break;
         }
      }
      BUILDING_QUESTION:;
		if (con_build_question(handle,AboutTheme,FromTheme)==FALSE)
		{
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_QUESTION_BUILDING_FAILED,"blad: proba zadania pytania nie powiodla sie"));
		}
   }

   // kasujemy fakt pod warunkiem, ze nie jest pamietana lista dowodowa
	if (handle->cache.facts!=NULL && handle->cache.last_list==NULL)
   {
    	dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
      handle->cache.facts=NULL;
	}

   return TRUE;
}

static int input_output_analysis(struct DeniseHandle *handle)
{
   /*
   funkcja zwraca liczbe wyrazow nie bedacych wskaznikami do preferowanej odpowiedzi
   */

   struct GrammarData *log;
   struct Fleksja *fleks,**allfleks;
   char temp[100],dane[100],posneg=TWIERDZENIE;
   int nr1,nr2,ile,poz,result=0;

   // identyfikujemy, czy to zdanie oznajmujace czy pytajace
   handle->syntax.input_type=get_sentence_type(handle->syntax.orig);
	if (handle->viewparams.preliminary_analysis==ON)
	{
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SENTENCE_TYPE,"wykryta forma wypowiedzi:"));
		if (handle->syntax.input_type==ZDANIE_OZNAJMUJACE) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SENTENCE_ANNOUNCE,"zdanie oznajmujace"));
		else DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SENTENCE_QUESTION,"zdanie pytajace"));
	}

   // rozpoznajemy oczekiwany format odpowiedzi
	if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_ANSWER,"oczekiwana forma odpowiedzi"));
   handle->syntax.prefered_output=NO_PREFERED; // wstepnie dowolna odpowiedz

   // pobieramy kolejne wyrazy
   START:
   if (strlen(handle->syntax.busy)==0) return result;
   ile=glo_count_parts(handle,handle->syntax.busy,' ');
   result=ile;

   for (nr1=0;nr1<ile;nr1++)
   {
      glo_get_part(handle->syntax.busy,nr1+1,temp,' ');
      // sprawdzamy, czy wyraz jest determinantem preferowanej odpowiedzi
      allfleks=dat_find_word(handle,handle->language.vocabulary,"",temp,"",0);
			fleks=*allfleks;
			free(allfleks);
      if (fleks==NULL) continue;

      poz=glo_get_part_position(1,fleks->odm_head->odmiany,temp,',');
      poz=glo_count_parts(handle,fleks->odm_head->odmiany,',')>1 ? poz : -1;
      if (poz!=-1) sprintf(dane,"%s%c",fleks->odm_head->opis,poz+'A'-1);
      else sprintf(dane,"%s",fleks->odm_head->opis);

      for (nr2=0;nr2<4;nr2++)
      {
         for (log=handle->language.lo4[nr2];log!=NULL;log=log->next)
         {
            // sprawdzamy zgodnosc
            glo_get_part(log->data,1,temp,'+');
            if (gra_wyrazenie_check(handle,&posneg,temp,dane)==0) continue;

            // jesli zgodne, to zapamietujemy oczekiwana forme odpowiedzi
            switch (nr2)
            {
               // PROVE_WITH_CON - dowod (zaimek nie jest kasowany)
               case 0:
						if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_PROVE,"oczekiwana odpowiedz w formie dowodu"));
                  handle->syntax.prefered_output=PREFERED_PROVE;
                  result--;
                  return result;
               // PROVE_WITHOUT_CON - dowod (zaimek jest kasowany)
               case 1:
                  if (ile>1) glo_delete_part_sep(handle,handle->syntax.busy,nr1+1,' ');
                  else handle->syntax.busy[0]='\0';
                  handle->syntax.prefered_output=PREFERED_PROVE;
                  goto START;
                  break;
               // RESULT_WITH_CON - tylko odpowiedz (zaimek nie jest kasowany)
               case 2:
						if (handle->viewparams.preliminary_analysis==ON) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_NO_PROVE,"oczekiwana odpowiedz bez dowodu"));
                  handle->syntax.prefered_output=PREFERED_RESULT;
                  result--;
                  return result;
               // RESULT_WITHOUT_CON - tylko odpowiedz (zaimek jest kasowany)
               case 3:
                  if (ile>1) glo_delete_part_sep(handle,handle->syntax.busy,nr1+1,' ');
                  else handle->syntax.busy[0]='\0';
                  handle->syntax.prefered_output=PREFERED_RESULT;
                  goto START;
                  break;
            }
         }
      }
   }

	if (handle->viewparams.preliminary_analysis==ON)
	{
		if (handle->syntax.prefered_output==PREFERED_PROVE) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_PROVE,"oczekiwana odpowiedz w formie dowodu"));
		else DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREFERED_NO_PROVE,"oczekiwana odpowiedz bez dowodu"));
	}
   return result;
}

static void make_busy(struct DeniseHandle* handle,char *orig,char *busy)
{
   char busy_char[]=".,!?:;*\\/-+#$^@[]{}()'";
   char space=' ';
   char nr1,nr2=0;
   char temp[MAX_LINE_LENGHT];

   strcpy(temp,orig);

   if (strlen(temp)==0)
   {
      busy[0]='\0';
      return;
   }

   // zamieniamy wszystkie znaki interpunkcyjne na spacje
   for (nr1=0;nr1<(int) strlen(temp);nr1++)
      if (strchr(busy_char,temp[nr1])!=0)
         temp[nr1]=space;

   // kasujemy wszystkie spacje na koncu
   for (nr1=strlen(temp)-1;nr1>=0;nr1--)
      if (temp[nr1]==space) temp[nr1]='\0';
      else break;

   // kasujemy wszystkie spacje na poczatku (przesuwamy pointer)
   for (nr1=0;nr1<(int) strlen(temp);nr1++)
      if (temp[nr1]!=space) break;

   // i likwidujemy podwojne spacje
	nr2=0;
   for (;nr1<(int) strlen(temp);nr1++)
   {
      if (temp[nr1]==space && nr1>0)
      {
         if (temp[nr1-1]!=space) busy[nr2++]=temp[nr1];
      }
      else busy[nr2++]=temp[nr1];
   }
   busy[nr2]='\0';

   // wszystkie duze znaki na male (nie rozpoznaje polskich! - dodano zabezpieczenie)
   for (nr1=0;nr1<(char) strlen(busy);nr1++)
	{
      if (isupper(busy[nr1]) && busy[nr1]>='A' && busy[nr1]<='Z') busy[nr1]=tolower(busy[nr1]);
	}

   ext_sentence_filter(handle,busy);
}

static int get_sentence_type(char *orig)
{
   int nr1;
   char space=' ';

   for (nr1=strlen(orig)-1;nr1>=0;nr1--)
   {
      if (orig[nr1]=='?') return ZDANIE_PYTAJACE;
      else if (orig[nr1]!=space) return ZDANIE_OZNAJMUJACE;
   }

   return ZDANIE_OZNAJMUJACE;
}

static BOOL is_identifier(struct DeniseHandle *handle,char *result)
{
   /*
   funkcja sprawdza, czy pobrana wypowiedz jest wprowadzeniem dodatkowego
   identyfikatora, jesli tak to zwraca TRUE, jesli nie FALSE,
   w result zawarte sa identyfikatory: wyraz+fleksja,wyraz+fleksja...

   algorytm:
   - zerujemy stan negacji,
   - pobieramy kolejne wyrazy odseparowane,
      - jesli to wyraz NEGACJI to zapamietujemy to - ustawiamy stan negacji,
      - jesli to wyraz inny, to:
         - sprawdzamy, czy jego fleksja jest zgodna z identyfikatorami,
            - jesli TAK, to dopisujemy wyraz,
            - jesli NIE, to RETURN FALSE,
         - zerujemy stan negacji.
   - RETURN TRUE,
   */

   struct GrammarData *gr_data;
   struct GrammarLo5Data *lo5_data;
   struct Fleksja *fleks,**allfleks;
   struct Odmiany *odm;
   char posneg=TWIERDZENIE;
   char temp1[100],temp2[100],dane[100],busy[100];
   int negation,nr1,ile,poz;

   result[0]='\0';

   // zerujemy stan negacji,
   negation=0;

   // pobieramy kolejne wyrazy odseparowane,
   make_busy(handle,handle->syntax.orig,busy);
   if (strlen(busy)==0) return FALSE;

   ile=glo_count_parts(handle,busy,' ');
   for (nr1=0;nr1<ile;nr1++)
   {
      glo_get_part(busy,nr1+1,temp1,' ');

      allfleks=dat_find_word(handle,handle->language.vocabulary,"",temp1,"",0);
			fleks=*allfleks;
			free(allfleks);
      if (fleks==NULL) continue;
      odm=dat_find_odm(handle,fleks->odm_head,temp1,"",0);

      poz=glo_get_part_position(1,odm->odmiany,temp1,',');
      poz=glo_count_parts(handle,odm->odmiany,',')>1 ? poz : -1;
      if (poz!=-1) sprintf(dane,"%s%c",odm->opis,poz+'A'-1);
      else sprintf(dane,"%s",odm->opis);

      // jesli to wyraz NEGACJI to zapamietujemy to - ustawiamy stan negacji,
      if (negation==0)
      {
         for (gr_data=handle->language.lo1;gr_data!=NULL;gr_data=gr_data->next)
         {
            if (strlen(gr_data->data)>strlen(dane)) continue;
            if (strncmp(gr_data->data,dane,strlen(dane))==0)
            {
               negation=1;
               break;
            }
         }
         if (negation==1) continue;
      }

      // jesli to wyraz inny, to:
      // sprawdzamy, czy jego fleksja jest zgodna z identyfikatorami,
      // przy czym sprawdzamy tylko zgodnosc fleksji,
      for (lo5_data=handle->language.lo5;lo5_data!=NULL;lo5_data=lo5_data->next)
      {
         if (gra_wyrazenie_check(handle,&posneg,lo5_data->czesc_mowy,dane)==0) continue;

         // jesli zgodne, to dopisujemy wyraz,
         if (result[0]!='\0')
         {
            sprintf(temp2,",");
            strcat(result,temp2);
         }
         if (negation==1)
         {
            sprintf(temp2,"!");
            strcat(result,temp2);
         }
         sprintf(temp2,"%s+%s",fleks->podst,dane);
         strcat(result,temp2);

         goto DALEJ;
      }
      // jesli NIE, to RETURN FALSE,
      result[0]='\0';
      return FALSE;

      DALEJ:
      // zerujemy stan negacji.
      negation=0;
   }

   if (result[0]=='\0') return FALSE;
   else return TRUE;
}

static int conv_get_conjunction(struct DeniseHandle *handle,char *expr)
{
	char temp1[100];
   int nr1,ile;

	ile=glo_count_parts(handle,expr,' ');
	for (nr1=0;nr1<ile;nr1++)
   {
   	glo_get_part(expr,nr1+1,temp1,' ');
      switch (atoi(temp1))
      {
			case CONV_AND: return CONV_AND; break;
			case CONV_OR: return CONV_OR; break;
      }
	}

	// w przypadku braku spojnika
   return CONV_OR;
}

static int conv_perform_action(int *action_table,int ile_action,int action)
{
	int nr1;

	for (nr1=0;nr1<ile_action;nr1++)
	{
		if (action_table[nr1]==action) return nr1;
	}
	return -1;
}

static BOOL meaning_analysis(struct DeniseHandle *handle)
{
	/*
	funkcja sprawdza, czy zdanie stanowi twierdzenie/pytanie o znaczeniach
	tozsamosciowych, i zapamietuje nowe znaczenie dla twierdzenia, oraz
	tylko odpowiada w przypadku pytania,

	algorytm:
	- pobieramy kolejne mozliwe ciagi rozpoczynajace zdanie tozsamowsciowe,
	- odszukujemy pobrany ciag w zdaniu,
	- jesli nie istnieje, to nast. ciag, w przeciwnym razie:
		- pobieramy kolejne mozliwe ciagi rozpoczynajace druga czesc tozsamosci
		- odszukujemy pobrany ciag w zdaniu ZA ciagiem rozpoczynajacym,
		- jesli nie istnieje, to nast. ciag, w przeciwnym razie:
			- odnajdujemy wyrazy podstawowe znalezionej tozsamosci,
			- sprawdzamy, czy taka tozsamosc istnieje w bazie:
			- jesli istnieje, to odpowiadamy, ze juz to wiemy i wychodzimy z funkcji TRUE,
			- jesli nie istnieje, to odpowiadamy, ze tego nie wiedzielismy, oraz
			  ew. zapamietujemy i wychodzimy z funkcji TRUE,
	*/

	struct Meaning *mean;
	struct Fleksja *fleks,**allfleks;
	char temp[200],temp1[200],temp2[200],basis1[200],basis2[200];
	int nr1,nr2,ile1,ile2;
	char *ptr1,*ptr2;

	if (handle->language.lo9==NULL) return FALSE;
	if (handle->language.lo9->mean2==NULL) return FALSE;

	if (handle->language.lo9->mean1==NULL) ile1=0;
	else ile1=glo_count_parts(handle,handle->language.lo9->mean1,',');

	ile2=glo_count_parts(handle,handle->language.lo9->mean2,',');
	if (ile2==0) return FALSE;

	for (nr1=0;;)
	{
		if (ile1>0)
		{
			// pobieramy kolejne mozliwe ciagi rozpoczynajace zdanie tozsamowsciowe,
			for (;nr1<ile1;nr1++)
			{
				glo_get_part(handle->language.lo9->mean1,nr1+1,temp1,',');
				temp1[strlen(temp1)-1]='\0';
				memmove(temp1,temp1+1,strlen(temp1));

				// odszukujemy pobrany ciag w zdaniu,
				if (temp1[0]=='\0') ptr1=handle->syntax.busy;
				else ptr1=strstr(handle->syntax.busy,temp1);

				// jesli nie istnieje, to nast. ciag, w przeciwnym razie skok do drugiej czesci
				if (ptr1!=NULL) {nr1++;goto PART2;}
			}
			// skoro zaden poczatek nie pasuje, to wyjscie z funkcji
			return FALSE;
		}
		else ptr1=handle->syntax.busy;

		PART2:;
		// pobieramy kolejne mozliwe ciagi rozpoczynajace druga czesc tozsamosci
		for (nr2=0;nr2<ile2;nr2++)
		{
			glo_get_part(handle->language.lo9->mean2,nr2+1,temp2,',');
			temp2[strlen(temp2)-1]='\0';
			memmove(temp2,temp2+1,strlen(temp2));
		
			// odszukujemy pobrany ciag w zdaniu ZA ciagiem rozpoczynajacym,
			ptr2=strstr(ptr1+strlen(temp1),temp2);

			// jesli nie istnieje, to nast. ciag, w przeciwnym razie ZNALEZIONO
			if (ptr2!=NULL)
			{
				sprintf(temp,"%.*s",ptr2-ptr1-strlen(temp1),ptr1+strlen(temp1));
				strcpy(temp1,temp);

				sprintf(temp,"%s",ptr2+strlen(temp2));
				strcpy(temp2,temp);

				if (temp1[0]==' ') memmove(temp1,temp1+1,strlen(temp1));
				if (temp2[0]==' ') memmove(temp2,temp2+1,strlen(temp2));
				if (temp1[strlen(temp1)-1]==' ') temp1[strlen(temp1)-1]='\0';
				if (temp2[strlen(temp2)-1]==' ') temp1[strlen(temp2)-1]='\0';

				// odnajdujemy wyrazy podstawowe znalezionej tozsamosci,
				ile1=glo_count_parts(handle,temp1,' ');
				basis1[0]='\0';
				for (nr1=0;nr1<ile1;nr1++)
				{
					glo_get_part(temp1,nr1+1,temp,' ');

					allfleks=dat_find_word(handle,handle->language.vocabulary,NULL,temp,NULL,0);
					fleks=*allfleks;
					free(allfleks);
					if (fleks==NULL)
					{
						DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_WORDS_NOT_FOUND,"nie odnaleziono wyrazu lub wyrazow w bazie fleksji"));
						return TRUE;
					}

					if (basis1[0]!='\0') strcat(basis1,",");
					strcat(basis1,fleks->podst);
				}
				ile1=glo_count_parts(handle,temp2,' ');
				basis2[0]='\0';
				for (nr1=0;nr1<ile1;nr1++)
				{
					glo_get_part(temp2,nr1+1,temp,' ');

					allfleks=dat_find_word(handle,handle->language.vocabulary,NULL,temp,NULL,0);
					fleks=*allfleks;
					free(allfleks);
					if (fleks==NULL)
					{
						DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_WORDS_NOT_FOUND,"nie odnaleziono wyrazu lub wyrazow w bazie fleksji"));
						return TRUE;
					}

					if (basis2[0]!='\0') strcat(basis2,",");
					strcat(basis2,fleks->podst);
				}

				// sprawdzamy, czy taka tozsamosc istnieje w bazie:
				for (mean=handle->meanings;mean!=NULL;mean=mean->next)
				{
					// sprawdzamy, czy wyrazy z grupy A pasuja do lewej lub do prawej czesci,
					// musi sie zgadzac liczba wyrazow w grupie z liczba wyrazow po lewej stronie
					if (glo_count_parts(handle,basis1,',')==mean->basisA_number)
					{
						for (nr1=0;nr1<mean->basisA_number;nr1++)
							if (glo_get_part_position(1,basis1,mean->basisA[nr1],',')==0) goto A_NOT_FOUND_ON_LEFT;

						// jesli TAK, to:
						// sprawdzamy, czy wyrazy z grupy B pasuja do przeciwnej czesci,
						if (glo_count_parts(handle,basis2,',')==mean->basisB_number)
						{
							for (nr1=0;nr1<mean->basisB_number;nr1++)
								if (glo_get_part_position(1,basis2,mean->basisB[nr1],',')==0) goto A_NOT_FOUND_ON_LEFT;

							// jesli TAK, znalezlismy tozsamosc,
							goto FOUNDED_TOZSAMOSC;
						}
						A_NOT_FOUND_ON_LEFT:;
					}

					// szukamy od prawej
					if (glo_count_parts(handle,basis1,',')==mean->basisB_number)
					{
						for (nr1=0;nr1<mean->basisB_number;nr1++)
							if (glo_get_part_position(1,basis1,mean->basisB[nr1],',')==0) goto A_NOT_FOUND_ON_RIGHT;

						// jesli TAK, to:
						// sprawdzamy, czy wyrazy z grupy B pasuja do przeciwnej czesci,
						if (glo_count_parts(handle,basis2,',')==mean->basisA_number)
						{
							for (nr1=0;nr1<mean->basisA_number;nr1++)
								if (glo_get_part_position(1,basis2,mean->basisA[nr1],',')==0) goto A_NOT_FOUND_ON_RIGHT;

							// jesli TAK, to znalezlismy tozsamosc
							goto FOUNDED_TOZSAMOSC;
						}
						A_NOT_FOUND_ON_RIGHT:;
					}
				}
//DeniseMessage(handle,"UNKNOWN: !'%s'+'%s'!",basis1,basis2);

				// odpowiedz
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_UNKNOWN,"brak danych"));

				if (handle->syntax.input_type==ZDANIE_OZNAJMUJACE)
				{
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_REMEMBER_NEW_FACT,"zapamietuje brakujace informacje"));

					mean=(struct Meaning *) malloc(sizeof(struct Meaning));
					mean->next=NULL;

					mean->basisA_number=glo_count_parts(handle,basis1,',');
					mean->basisA=(char **) malloc(mean->basisA_number*sizeof(char *));
					for (nr1=0;nr1<mean->basisA_number;nr1++)
					{
						glo_get_part(basis1,nr1+1,temp,',');
						mean->basisA[nr1]=malloc(strlen(temp)+1);
						memcpy(mean->basisA[nr1],temp,strlen(temp)+1);
					}

					mean->basisB_number=glo_count_parts(handle,basis2,',');
					mean->basisB=(char **) malloc(mean->basisB_number*sizeof(char *));
					for (nr1=0;nr1<mean->basisB_number;nr1++)
					{
						glo_get_part(basis2,nr1+1,temp,',');
						mean->basisB[nr1]=malloc(strlen(temp)+1);
						memcpy(mean->basisB[nr1],temp,strlen(temp)+1);
					}

					// dodanie do listy
					if (handle->meanings!=NULL) mean->next=handle->meanings;
					handle->meanings=mean;
				}
				return TRUE;

				// jesli taka tozsamosc istnieje juz
				FOUNDED_TOZSAMOSC:

				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_ANSWER_ITS_TRUE,"to prawda"));
//DeniseMessage(handle,"FOUNDED: !'%s'+'%s'!",basis1,basis2);
				return TRUE;
			}
		}

		// wyjscie z petli konieczne ze wzgledu na mozliwosc braku sekcji 1 w mean
		if (handle->language.lo9->mean1==NULL) break;
	}

	return FALSE;
}
